import pathlib


S3_BUCKET_URI = "s3://fast-datasets/{ds}"

# Read Only Access
AWS_ACCESS_KEY_ID = "AKIAVXFFSLWHED2I7WGB"
AWS_SECRET_ACCESS_KEY = "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"

# Download Temp Path
TMP_PATH = "/tmp/{ds}"

# 默认数据集存储地址
DEFAULT_DATASET_PATH = pathlib.Path('/Users/zhangchunyang/.cache/datasetstation/datasets/')

# Divide String
D_S = "=" * 50



import cufflinks as cf
import plotly.graph_objects as go
import plotly.express as px
from wordcloud import WordCloud
import matplotlib.pyplot as plt


cf.go_offline()
cf.set_config_file(offline=False, world_readable=True)


def length_distribution(dataset, text_column_name, quantile=0.8):
    """文本长度分布"""
    dataframe = dataset.to_pandas()
    dataframe["text_len"] = dataframe[text_column_name].map(
        lambda x: len(x) if x else 0
    )
    len_count_df = dataframe.groupby("text_len", as_index=False).count()
    max_example_count = len_count_df[text_column_name].max()
    max_len = len_count_df["text_len"].max()
    len_count_df = len_count_df.rename(
        {"text_len": "文本长度", text_column_name: "样本个数"}, axis="columns"
    )

    # 适合截断长度计算
    quantile_idx = int(len(dataframe) * quantile)
    sorted_len_df = dataframe.sort_values(by=["text_len"])
    v = sorted_len_df.reset_index(drop=True)
    suitable_length = v.loc[quantile_idx - 1]

    # 画柱状图
    fig = px.bar(len_count_df, x="文本长度", y="样本个数", text_auto=True)
    fig.add_shape(
        type="line",
        x0=suitable_length["text_len"],
        y0=-10,
        x1=suitable_length["text_len"],
        y1=max_example_count + 10,
        line=dict(color="#7D9D9C", width=1),
    )

    # 画提示性文字
    fig.add_trace(
        go.Scatter(
            x=[max_len / 2],
            y=[(max_example_count + 10) / 2],
            text=[
                "文本最大长度 {}\n\r占比全数据集 {}".format(suitable_length["text_len"], quantile),
            ],
            mode="text",
            fillcolor="#7D9D9C",
        )
    )

    # 更新 layout
    layout = go.Layout(paper_bgcolor="rgba(0,0,0,0)", plot_bgcolor="rgba(0,0,0,0)")
    fig.update_layout(layout)
    fig.show()


#用来正常显示中文
plt.rcParams["font.sans-serif"]=["SimHei"]
#用来正常显示负号
plt.rcParams["axes.unicode_minus"]=False

def wordcloud(texts):
    """绘制词云"""
    wordcloud = WordCloud(
        background_color="white",
        font_path="/System/Library/Fonts/PingFang.ttc",
        width=1400,
        height=800
    ).generate(" ".join(texts))
    plt.axis("off")  
    plt.imshow(wordcloud)  
    plt.show()


# 字频率分布

# 词频率分布

# 词性频率分布



from datasetstation.util.regex import absence


# ---------------------------------------------------------------------
# 手机号码
CELL_PHONE_PATTERN = r'(?<=[^\d])(((\+86)?([- ])?)?((1[3-9][0-9]))([- ])?\d{4}([- ])?\d{4})(?=[^\d])'
# 该规则用于抽取与判定手机号的归属地，即抽取前三位、中间4位
CELL_PHONE_CHECK_PATTERN = r'((1[3-9][0-9]))([- ])?\d{4}([- ])?\d{4}'

# 手机|电话 号码前缀，例如：`Tel: 18902437922`，用于删除前缀
PHONE_PREFIX_PATTERN = r'(([tT](el(ephone)?|EL(EPHONE)?)|[cC](ell(phone)?|ELL(PHONE)?)|' \
                       r'((联系)?电[ \t\u3000]*话|手[ \t\u3000]*机)(号(码)?)?)(:|：)?[\t \u3000]*)' \
                       r'(?=[^:： \t\u3000])'

# ---------------------------------------------------------------------
# 中文字符正则
ANCIENT_CHINESE_CHAR_PATTERN = '[一-龥㐀-䶵]'  # 在 gb13000.1 基础上扩展 6582 个古汉字，共 27484 个汉字
# gb13000.1 收录的汉字 20902 个，但其中有很多不常用字，在 chinese_char_dictionary_loader 有说明
CHINESE_CHAR_PATTERN = '[一-龥]'

# ---------------------------------------------------------------------
# 国内固定电话号码
LANDLINE_PHONE_PATTERN = r'(?<=[^\d])(([\(（])?0\d{2,3}[\)） —-]{1,2}\d{7,8}|\d{3,4}[ -]\d{3,4}[ -]\d{4})(?=[^\d])'
LANDLINE_PHONE_CHECK_PATTERN = r'(([\(（])?0\d{2,3}[\)） —-]{1,2}\d{7,8}|\d{3,4}[ -]\d{3,4}[ -]\d{4})'
# 用分隔符，找到靠前的区号
LANDLINE_PHONE_AREA_CODE_PATTERN = r'(0\d{2,3})[\)） —-]'

# ---------------------------------------------------------------------
# 电子邮箱
# 邮箱中，允许中文等字符存在。但是，中文字符、#、* 会对结果造成较大干扰故忽略
EMAIL_PATTERN = r'(?<=[^0-9a-zA-Z.\-])' \
                r'([a-zA-Z0-9_.-]+@[a-zA-Z0-9_.-]+(?:\.[a-zA-Z0-9_.-]+)*\.[a-zA-Z0-9]{2,6})' \
                r'(?=[^0-9a-zA-Z.\-])'
# 抽取邮箱的域名
EMAIL_DOMAIN_PATTERN = r'(?<=@)([0-9a-zA-Z]+)(?=\.)'

# 抽取邮箱的前缀，一般为：`email: 32e8u9u@gmail.com`，用于删除前缀
EMAIL_PREFIX_PATTERN = r'(([eE](\-|—)?(mail|MAIL)|(电子)?邮箱)(:|：)?[\t \u3000]*)' \
                       r'(?=[^:： \t\u3000])'

# ---------------------------------------------------------------------
# 转义符号
ESCAPE_CHAR_PATTERN = '\t\n\a\b\f\r\v'

# ---------------------------------------------------------------------
# 异常字符
# - 字节编码，包括单字节与多字节 unicode 编码，均有大量的字符无法正常显示，且
# 无对应的打印文本，这部分字符需要被剔除。
# - 此外，仍有大量的可打印字符，由于出现概率极低，且对中文处理作用极小，可以删除。

# 一、单字节字符：
# 即一个字节表示的字符，\x00~\xff，其中有一部分转义字符不打印，且非 # \t\n\r。
# 因此需要作为异常字符剔除掉，主要包括：\x00~\x08，\x0e~\x1f，\x7f~\x9f，\xa1~\xff
# 剩余的单字节字符参考 ascii 码表
ASCII_EXCEPTION_PATTERN = '[^\x09-\x0d\x20-\x7e\xa0£¥©®°±×÷]'

# 二、UNICODE 字符
# 即多个字节组成的字符，其中，囊括了各种语言的各种符号，这里我们只关心常用中文字符
# 以及相应的常用符号，单字节符号、标点符号等。而日文、俄文、拉丁、希腊、数学公式、
# 物理单位等符号 绝大多数不常用的都被丢弃。其中 㐀-䶵 指的是另一个汉字字符集
# 仅保留了常用符号，数字标识，如 ① 等
UNICODE_EXCEPTION_PATTERN = '[^‐-”•·・…‰※℃℉Ⅰ-ⅹ①-⒛\u3000-】〔-〞㈠-㈩一-龥﹐-﹫！-～￠￡￥]'
EXCEPTION_PATTERN = ASCII_EXCEPTION_PATTERN[:-1] + UNICODE_EXCEPTION_PATTERN[2:]

# ---------------------------------------------------------------------
# 全角字母、数字、空格正则
FULL_ANGLE_ALPHABET = 'ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ０１２３４５６７８９　'
HALF_ANGLE_ALPHABET = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 '

# ---------------------------------------------------------------------
# HTML 标签
HTML_TAG_PATTERN = '<[^<\u4E00-\u9FA5，。；！？、“”‘’（）—《》…●]+?>'

# ---------------------------------------------------------------------
# 中国身份证号
# 身份证共计18位，1,2位省份，3,4位城市，5,6位县区码，7~14位为出生日期，最后一位为校验码，做了严格限定
ID_CARD_PATTERN = r'(?<=[^0-9a-zA-Z])' \
                  r'((1[1-5]|2[1-3]|3[1-7]|4[1-6]|5[0-4]|6[1-5]|71|81|82|91)' \
                  r'(0[0-9]|1[0-9]|2[0-9]|3[0-4]|4[0-3]|5[1-3]|90)' \
                  r'(0[0-9]|1[0-9]|2[0-9]|3[0-9]|4[0-3]|5[1-7]|6[1-4]|7[1-4]|8[1-7])' \
                  r'(18|19|20)\d{2}(0[1-9]|1[0-2])(0[1-9]|[12][0-9]|3[01])' \
                  r'\d{3}[0-9xX])' \
                  r'(?=[^0-9a-zA-Z])'
ID_CARD_CHECK_PATTERN = r'^(1[1-5]|2[1-3]|3[1-7]|4[1-6]|5[0-4]|6[1-5]|71|81|82|91)' \
                        r'(0[0-9]|1[0-9]|2[0-9]|3[0-4]|4[0-3]|5[1-3]|90)' \
                        r'(0[0-9]|1[0-9]|2[0-9]|3[0-9]|4[0-3]|5[1-7]|6[1-4]|7[1-4]|8[1-7])' \
                        r'(19|20)\d{2}(0[1-9]|1[0-2])(0[1-9]|[12][0-9]|3[01])\d{3}[0-9xX]$'

# ---------------------------------------------------------------------
# IP地址
# 0~255.0~255.0~255.0~255
IP_SINGLE = r'(25[0-5]|2[0-4]\d|[0-1]\d{2}|[1-9]?\d)'
IP_ADDRESS_PATTERN = ''.join(
    [r'(?<=[^0-9])(', IP_SINGLE, r'\.', IP_SINGLE, r'\.', IP_SINGLE, r'\.', IP_SINGLE, ')(?=[^0-9])'])

# ---------------------------------------------------------------------
# 地名表达式
# 除中国地名表外，还有大量的地名未进行识别，采用正则匹配。然而仍然有一些特例：
# “城”字结尾的名词有不少房地产楼盘
LOCATION_CONTINENT_PATTERN = '(亚|欧|非|北美|中(南)?美|南美|拉丁美|南极|大洋)洲'  # 大洲
LOCATION_OCEAN_PATTERN = '(东|西|南|北)?(太平|大西|印度|北冰)洋'  # 大洋
LOCATION_INTER_REGION_PATTERN = '((东|西|南|北|中|东南|中北)亚|(东|西|南|北|中)欧|(东|西|北)非|拉美|北美|南美|(中|近|远)东)'  # 国际区域名词
LOCATION_KEYWORDS_PATTERN = CHINESE_CHAR_PATTERN + '+[县市镇村区山州路河城湖岛港江省湾乡街庄堡国寺桥溪岭海郡]'  # 尾缀词地名

# 各种不同类型的地名可以自由添加组合
LOCATION_PATTERN = '^(' + '|'.join(
    [LOCATION_CONTINENT_PATTERN, LOCATION_OCEAN_PATTERN,
     LOCATION_INTER_REGION_PATTERN, LOCATION_KEYWORDS_PATTERN]) + ')$'

# ---------------------------------------------------------------------
# 金额表达式 
# 裁判文书金额解析的正则
# '(\d[,，\d]*\.?\d+|[一二三四五六七八九十壹贰叁肆伍陆柒捌玖拾两][一二三四五六七八九十壹贰叁肆伍陆柒捌玖拾两〇O零百千万亿佰仟]*?)
# [百千万亿佰仟]*?美?元
# ((\d[,.，\d]*|[一二三四五六七八九十壹贰叁肆伍陆柒捌玖拾两零][一二三四五六七八九十壹贰叁肆伍陆柒捌玖拾两〇O零百千万亿佰>仟]*?)角)?
# ((\d[,.\d]*|[一二三四五六七八九十壹贰叁肆伍陆柒捌玖拾两零][一二三四五六七八九十壹贰叁肆伍陆柒捌玖拾两〇O零百千万亿佰仟]*?)分)?'

# 存在一定的错误金额字符串依然能够解析并通过的情况
CHINESE_NUM = '[一二三四五六七八九壹贰叁弎仨肆伍陆柒捌玖俩两零]'  # 金额数字
CHINESE_UNIT = '[〇O零十百千万亿兆拾佰仟萬億]'  # 金额数字单位
CURRENCY_CASE = r'(块(钱)?|元((人民|港|日|澳|韩|(新)?台)币)?|(人民|港|日|澳|韩|(新)?台)币|圆(整)?|' \
                r'(美|港|澳门|日|韩|缅|马|新加坡|欧|加|新西兰|澳|澳大利亚)元|美(金|刀)|英镑|马克|法郎|卢布|泰铢)'

CHI_N = CHINESE_NUM
CHI_U = CHINESE_UNIT

# 标准金额数字格式 7,129,012.02元
MONEY_PATTERN_1 = r'((\d{1,3}([,，]\d{1,3})*(\.\d{0,2})?)' + CURRENCY_CASE + ')'
# 纯数字格式 340000.0元
MONEY_PATTERN_2 = r'((\d{1,12}(\.\d{0,2})?)' + CURRENCY_CASE + ')'
# 中文金额格式 一万二千三百四十五
CHINESE_MONEY_PATTERN = ''.join(['(((', CHI_N, '?', CHI_U, '{1,2})*', CHI_N, '?)'])
# 正式文本中文金额格式 一万二千三百四十五元
MONEY_PATTERN_3 = CHINESE_MONEY_PATTERN + CURRENCY_CASE + '(' + CHI_N + '[角|毛])?(' + CHI_N + '分)?)'
# 口语文本中文金额格式 “三十五块八毛”，但不允许 “三十五块” 或 “三十五块八” 出现：有歧义
MONEY_PATTERN_4 = CHINESE_MONEY_PATTERN + '(块)' + '(' + CHI_N + '[角|毛])(' + CHI_N + '分)?)'
# 数字+汉字单位格式 9300万元  1.2万元  9佰元
MONEY_PATTERN_5 = r'(\d{1,4}(\.\d{0,4})?' + CHI_U + CURRENCY_CASE + ')'

MONEY_PATTERN = '(' + '|'.join(
    [MONEY_PATTERN_1, MONEY_PATTERN_2,
     MONEY_PATTERN_3, MONEY_PATTERN_4, MONEY_PATTERN_5]) + ')'

# ---------------------------------------------------------------------
# 中文括号，用于提取括号内容，或删除
PARENTHESES_PATTERN = '{}「」[]【】()（）<>《》〈〉『』〔〕｛｝＜＞〖〗'


# ---------------------------------------------------------------------
# 标点符号
PUNCTUATION_PATTERN = ''

# ---------------------------------------------------------------------
# 腾讯QQ号
QQ_PATTERN = '(?<=[^0-9])([1-9][0-9]{5,10})(?=[^0-9])'
STRICT_QQ_PATTERN = r'(qq|QQ|\+q|\+Q|加q|加Q|q号|Q号)'
# STRICT_QQ_PATTERN = '([qQ]{2}[:： ]{0,5}[1-9][0-9]{5,10})(?=[^0-9])'

# ---------------------------------------------------------------------
# 腾讯微信号
WECHAT_ID_PATTERN = '(?<=[^\da-zA-Z_\-])([a-zA-Z][a-zA-Z\d_\-]{5,19})(?=[^\da-zA-Z_\-])'
STRICT_WECHAT_ID_PATTERN = r'(微信(号)?(:|：)?|wx(:|：)|wechat(:|：))'
# STRICT_QQ_PATTERN = '([qQ]{2}[:： ]{0,5}[1-9][0-9]{5,10})(?=[^0-9])'

# ---------------------------------------------------------------------
# 冗余字符处理
# 文本中有连续的 “哈哈哈哈哈” 等字符串，需要删除冗余字符串，返回为 “哈”
REDUNDANT_PATTERN = ' -\t\n啊哈呀~\u3000\xa0•·・'

# ---------------------------------------------------------------------
# 人名百家姓
# 用于短暂处理和过滤人名等
SINGLE_CHINESE_FAMILY_NAME = \
    '赵钱孙李周吴郑王冯陈褚卫蒋沈韩杨朱秦尤许何吕施张孔曹严华金魏陶姜戚谢邹喻柏水窦章云苏潘葛奚范彭郎鲁韦昌' \
    '马苗凤花方俞任袁柳酆鲍史唐费廉岑薛雷贺倪汤滕殷罗毕郝邬安常乐于时傅皮卞齐康伍余元卜顾孟平黄和穆萧尹姚邵' \
    '湛汪祁毛禹狄米贝明臧计伏成戴谈宋茅庞熊纪舒屈项祝董梁杜阮蓝闵季麻强贾路娄危江童颜郭梅盛林刁钟徐邱骆高夏' \
    '蔡田樊胡凌霍虞万支柯昝管卢莫经房裘缪干解应宗丁宣贲邓郁单杭洪包诸左石崔吉钮龚程嵇邢滑裴陆荣翁荀羊於惠甄' \
    '曲家封芮羿储靳汲邴糜松井段富巫乌焦巴弓牧隗山谷车侯宓蓬全郗班仰秋仲伊宫宁仇栾暴甘钭厉戎祖武符刘景詹束龙' \
    '叶幸司韶郜黎蓟薄印宿白怀蒲邰从鄂索咸籍赖卓蔺屠蒙池乔阴胥能苍双闻莘党翟谭贡劳逄姬申扶堵冉宰郦雍郤璩桑桂' \
    '濮牛寿通边扈燕冀郏浦尚农温别庄晏柴瞿阎充慕连茹习宦艾鱼容向古易慎戈廖庾终暨居衡步都耿满弘匡国文寇广禄阙' \
    '东欧殳沃利蔚越夔隆师巩厍聂晁勾敖融冷訾辛阚那简饶空曾毋沙乜养鞠须丰巢关蒯相查後荆红游竺权逯盖益桓公' \
    '仉督晋楚闫法汝鄢涂钦归海岳帅缑亢况后有琴商牟佘佴伯赏墨哈谯笪年爱阳佟言福'

# 此处为：将存在歧义的姓氏去除掉，剩余的姓氏字，一旦出现，则大概率为姓名表示
SINGLE_CHINESE_FAMILY_NAME = \
    '赵李吴郑王冯陈褚蒋沈韩杨朱秦尤许何吕张孔曹严魏陶姜戚邹喻窦潘葛奚范彭郎鲁韦' \
    '俞袁酆鲍史廉岑薛倪滕殷罗郝邬傅卞康伍卜顾孟穆萧尹姚邵' \
    '湛汪祁禹狄臧宋茅董梁杜阮闵贾娄郭盛刁钟徐邱骆' \
    '蔡樊凌霍虞柯昝卢莫裘缪丁贲邓郁崔龚嵇邢裴翁荀於惠甄' \
    '芮羿储靳汲邴糜弓隗侯宓蓬郗仲伊栾钭刘詹' \
    '韶郜黎蓟薄蒲邰鄂蔺乔胥莘翟谭贡逄姬冉郦雍郤璩' \
    '濮扈冀郏尚晏瞿阎慕茹宦艾易慎戈廖庾暨衡耿弘匡寇禄阙' \
    '殳沃夔厍聂晁敖融訾阚饶毋乜鞠巢蒯後竺逯桓' \
    '仉晋楚闫汝鄢涂钦缑亢牟佘佴赏谯笪佟'

TWO_CHAR_CHINESE_FAMILY_NAME = \
    '万俟|司马|上官|欧阳|夏侯|诸葛|闻人|东方|赫连|皇甫|尉迟|公羊|澹台|公冶|宗政|濮阳|淳于|单于|太叔|申屠|' \
    '公孙|仲孙|轩辕|令狐|钟离|宇文|长孙|慕容|鲜于|闾丘|司徒|司空|亓官|司寇|子车|颛孙|端木|巫马|公西|漆雕|' \
    '乐正|壤驷|公良|拓跋|夹谷|宰父|谷梁|段干|百里|东郭|南门|呼延|羊舌|微生|梁丘|左丘|东门|西门|南宫|第五'

CHINESE_FAMILY_NAME = '(' + '|'.join(SINGLE_CHINESE_FAMILY_NAME) + \
                      '|' + TWO_CHAR_CHINESE_FAMILY_NAME + ')'

# ---------------------------------------------------------------------
# 纯数字格式，用于过滤停用词时，过滤掉纯数字（包括汉字数字）
# 融合了百分比格式、序数词，形容词（数*、*余等），负数，数字范围等，还差分数表
# 示未添加，如 “三十分之一”
BASE_NUMBER_PATTERN = '[' + CHINESE_NUM[1:-1] + CHINESE_UNIT[1:-1] + r'点\d\%％\.\,．多余几]+'
NUMBER_PATTERN = r'^((十|百|千|万)分之|第|数|好|\-)?' + BASE_NUMBER_PATTERN + r'([\~\-～－至]?' + BASE_NUMBER_PATTERN + ')?(多|余)?$'

# 纯数字格式（不包括汉字数字）
# 融合了整数、小数、百分比等
PURE_NUMBER_PATTERN = r'\-?\d+(\.\d+)?(%|％)?'

# ---------------------------------------------------------------------
# 时间词汇，用于停用词过滤时，将时间词汇过滤掉
# 1. 时间格式仅用于滤除具体确切的时间点和时间段，如“2019年6月30日”，“第一季度”，
#   “18:30:51”，“3~4月份”，“清晨”，“年前” 等等，此类词汇描述了具体的时间，在语言中
#   一般作为时间状语存在，因此在停用词滤除中，需要将该部分词汇滤除。
# 2. 但不滤除模糊的时间范围，如“三十年”，“六七个月”，“十周”，“四日” 等等，这些时间
#   描述了一个模糊的时间段，并没有确切的指代，在语言中一般做宾语，补语，主语等，因此
#   在停用词滤除中，一般不将此类词汇滤除。
# 3. 有些词汇含义指代不明，如“三十一日”，具体指某月 31日，还是31天的时间，并不确切，
#   此时不予滤除。
# 4. 节日名称不予滤除，如“圣诞节”、“除夕夜”，尽管其指示具体的时间点，但是一般做名词性
#   成分，因此不予滤除。

# 时分秒格式
HO_N = r'([01]?\d|2[01234])'  # 时 数字格式
MI_N = r'[012345]?\d'  # 分 数字格式
SE_N = r'[012345]?\d'  # 秒 数字格式
HMS_GAP = '[:：]'
HMS_PATTERN_1 = '^(' + HO_N + HMS_GAP + MI_N + '(' + HMS_GAP + SE_N + ')?)$'  # 纯数字格式时分秒，或时分
HMS_PATTERN_2 = '^(' + HO_N + '(点|时|小时)(' + MI_N + '分(钟)?(' + SE_N + '秒(钟)?)?)?)$'  # 带汉字时分秒
HMS_PATTERN_3 = '^(' + HMS_PATTERN_1 + r'[\-\~～—]{1,2}' + HMS_PATTERN_1 + ')$'  # 时间段
# HMS_PATTERN_4 = '^([012]?\d点)$'  # 有一定前提条件，即前后必须也有时间词汇

# 年月日格式
YE_N = r'[12]?\d{2,3}'  # 年份数字格式
MO_N = r'([0]?\d|1[012])'  # 月份数字格式
MO_C = r'(元|正|腊|一|二|三|四|五|六|七|八|九|十(一|二)?)'  # 月份汉字格式
DA_N = r'([012]?\d|3[01])'  # 日数字格式
YMD_GAP = r'[\-\~— ～\.]{1,2}'
SPAN_GAP = r'[\~\-～－至]'

YMD_PATTERN_1 = '^((公元(前)?)?' + YE_N + '年(初|底|中)?)?((' + MO_N + '|' + MO_C + ')月(份|底|初)?)?(' + DA_N + '[日号])?$'  # 带汉字年月日
YMD_PATTERN_2 = '^(' + YE_N + YMD_GAP + MO_N + '(' + YMD_GAP + DA_N + ')?)$'  # 纯数字年月日，或年月
YMD_PATTERN_3 = '^(' + MO_N + YMD_GAP + DA_N + '(' + YMD_GAP + YE_N + ')?)$'  # 纯数字月日年，或月日
YMD_PATTERN_4 = '^((公元(前)?)?' + r'(([12]?\d|(二)?十(一|二|三|四|五|六|七|八|九)?)世纪)?((\d0|(一|二|三|四|五|六|七|八|九)十)年代)?(初|末)?' + ')$'  # 世纪，年代
YMD_PATTERN_5 = '^(一|二|三|四|五|六|七|八|九|零|〇|○|0){4}年$'

# 年月日-时分秒合并格式
YMD_HMS_PATTERN = '^(' + YMD_PATTERN_2[1:-1] + r'([\-\~\—～]{1,2})?' + HMS_PATTERN_1[1:-1] + ')$'

# 农历日期
LUNAR_PATTERN = '((闰)?(元|正|腊|一|二|三|四|五|六|七|八|九|十(一|二)?)月|大年)(初(一|二|三|四|五|六|七|八|九|十)|(一|二|三|四|五|六|七|八|九|十){2,3})'

# 时间段
# 年时间段
YEAR_SPAN_PATTERN = '^' + YE_N + SPAN_GAP + YE_N + '年(代)?$'
# 月时间段
MONTH_SPAN_PATTERN = '^' + MO_N + SPAN_GAP + MO_N + '月(份)?$'
# 日时间段
DAY_SPAN_PATTERN = '^' + DA_N + SPAN_GAP + DA_N + '日$'

# 季节格式
SEASON_PATTERN = '((春|夏|秋|冬){1,2}(季|天|日)|(第)?(一|二|三|四)(季度)(末)?)'

# 星期格式
WEEK_PATTERN = '((上(半)?|下(半)?|这|本|前|今|当|上上|下下)?(星期|周)(六日|一|二|三|四|五|六|日|七|天|末|初)?(时)?)'

# 常见时间短语
COMMON_TIME_PATTERN_1 = '^(年|月|日|时)$'
COMMON_YEAR_PATTERN = r'(昔|翌|头(一|两|几|些)?|(大)?前(一|半|两|几|些)?|近(一|两|几|些)?|这(一|两|几|些)?|那(一|两|几|些)?|上(半)?|下(半)?|(大)?后(一|半|两|几|些)?|同|当|早(一|两|几|些)?|每|去|今|往|本|次|明|明后)?年(中|度|初|前|末|底|终|内)?'
COMMON_SEASON_PATTERN = r'(本|上|下|这)'  # 季度
COMMON_MONTH_PATTERN = r'(下(个)?|首(个)?|前(两|几)?|上(个)?|这(个)?|次|这(些|个)?|那(些|个)?|上半(个)?|下半(个)?|同|本|当|每)?月(份|中|度|初|末|底)?'
COMMON_DAY_PATTERN = r'(昔|首|前(一|两|几|些)?|翌|昨|次|今|往|明|平|即|往|半|旧|近(一|两|几|些)?|后(一|两|几|些)?|这(一|两|几|些)?|那(一|两|几|些)?|上半|下半|同|当|每(一)?)?(天|日)(前|后)?'
COMMON_TIME_PATTERN_2 = '(下|中|上)(午|旬)|近(期|日)|此前'
COMMON_TIME_PATTERN_3 = '(晚|早)(上|间)'
COMMON_TIME_PATTERN_4 = '(深|每|昨|前|今|午|后|半|上半|下半|春|当|夏|秋|冬)?夜(里|晚|间)?'
COMMON_TIME_PATTERN_5 = '(今|傍|昨|当)晚'
COMMON_TIME_PATTERN_6 = '(早|凌|今|清)晨|黎明'
COMMON_TIME_PATTERN_7 = '午(后|时)'

# 各类型的时间正则汇总，可根据需要进行增删
TIME_PATTERN = '(' + '|'.join(
    [COMMON_TIME_PATTERN_1,
     COMMON_YEAR_PATTERN, COMMON_SEASON_PATTERN,
     COMMON_MONTH_PATTERN, COMMON_DAY_PATTERN,
     COMMON_TIME_PATTERN_2, COMMON_TIME_PATTERN_3, COMMON_TIME_PATTERN_4,
     COMMON_TIME_PATTERN_5, COMMON_TIME_PATTERN_6, COMMON_TIME_PATTERN_7,
     WEEK_PATTERN, LUNAR_PATTERN, YMD_HMS_PATTERN, SEASON_PATTERN,
     YMD_PATTERN_1, YMD_PATTERN_2, YMD_PATTERN_3, YMD_PATTERN_4,
     YMD_PATTERN_5,
     HMS_PATTERN_1, HMS_PATTERN_2, HMS_PATTERN_3,
     YEAR_SPAN_PATTERN, MONTH_SPAN_PATTERN, DAY_SPAN_PATTERN, ]) + ')'

# ---------------------------------------------------------------------
# URL
URL_PATTERN = r'(?<=[^.])((?:(?:https?|ftp|file)://|(?<![a-zA-Z\-\.])www\.)' \
              r'[\-A-Za-z0-9\+&@\(\)#/%\?=\~_|!:\,\.\;]+[\-A-Za-z0-9\+&@#/%=\~_\|])' \
              r'(?=[<\u4E00-\u9FA5￥，。；！？、“”‘’>（）—《》…● \t\n])'


#######################################################################
# 针对 time_parser 的正则字符串
# 字符串操作
# 年
LIMIT_YEAR_STRING = r'(前(一)?|今|明|去|同|当|后|大前|本|次|上(一)?|这(一)?)年'
LUNAR_YEAR_STRING = r'([一二三四五六七八九零〇]{2}|[一二三四五六七八九零〇]{4}|[12]\d{3}|\d{2})年'
YEAR_STRING = r'([12]?\d{2,3}|[一二三四五六七八九零〇]{2,4})年'

# 月
MONTH_NUM_STRING = r'(1[012]|[0]?[1-9]|十[一二]|[一二三四五六七八九十])'  # 1~12 std month num
MONTH_STRING = MONTH_NUM_STRING + r'月(份)?'
MONTH_NUM_STRING = MONTH_NUM_STRING[:-2] + r'两])'  # 1~12 order month num
BLUR_MONTH_STRING = r'(初|[一]开年|伊始|末|尾|终|底|[上下]半年|[暑寒][假期]|[前中后]期)'
LUNAR_MONTH_STRING = r'(闰)?([正一二三四五六七八九十冬腊]|十[一二]|[1-9]|1[012])月'
LIMIT_MONTH_STRING = r'([下上]((一)?个)?|同|本|当|次|这)月'
SELF_EVI_LUNAR_MONTH_STRING = r'((闰)?[正冬腊]|闰([一二三四五六七八九十]|十[一二]|[1-9]|1[012]))月'

# 周
WEEK_NUM_STRING = r'[一二两三四五六七八九十0-9]{1,3}'  # 1~52
WEEK_STRING = r'(周|星期|礼拜)'

# 日
DAY_NUM_STRING = r'(([12]\d|3[01]|[0]?[1-9])|([一二]?十)?[一二三四五六七八九]|(三十)?[一]|[二三]?十)'  # 1~31
DAY_STRING = DAY_NUM_STRING + r'[日号]'
BLUR_DAY_STRING = r'([上中下]旬|初|中|底|末)'
# 允许 `初8` 阿拉伯数字出现，但不允许 `廿2`、`23` 等作为农历`日`
LUNAR_SOLAR_DAY_STRING = r'((初|(二)?十|廿)[一二三四五六七八九]|[初二三]十|初([1-9]|10)|[12]\d|3[01]|[0]?[1-9])'
LUNAR_DAY_STRING = r'((初|(二)?十|廿)[一二三四五六七八九]|[初二三]十|初([1-9]|10))'
SELF_EVI_LUNAR_DAY_STRING = r'([初廿]([一二三四五六七八九十1-9]|10))'

# 时
HOUR_STRING = r'((十)?[一两二三四五六七八九]|[零〇十]|二十[一二三四]?|[01]?\d|2[01234])[时点]'
BLUR_HOUR_STRING = r'(凌晨|白天|清[晨|早]|黎明|一(大)?早|早[晨上]?|[上中下]午|午后|(傍)?晚[间上]?|[深半午]?夜[里间]?|[上下前后]半夜)'

# 分、秒
MIN_SEC_STRING = r'((零|〇|[一二三四五]?十)[一二三四五六七八九]|[二三四五]?十|[012345]?\d)'

# seg
I = '|'
LU = r'([农阴]历)'
LU_A = absence(LU)

# appendix
TIME_POINT_SUFFIX = r'(左右|许|前后)'
TIME_SPAN_SUFFIX = r'((之)?间)'
# TIME_DELTA_SUFFIX = r''

# 节气
SOLAR_TERM_STRING = r'(立春|雨水|惊蛰|春分|清明|谷雨|立夏|小满|芒种|夏至|小暑|大暑|'\
    r'立秋|处暑|白露|秋分|寒露|霜降|立冬|小雪|大雪|冬至|小寒|大寒)'

# 固定公历节日
FIXED_SOLAR_FESTIVAL = r'((元旦|十一)|(三八|五一|六一|七一|八一|国庆|圣诞)(节)?|'\
    r'((三八)?妇女|女神|植树|(五一)?劳动|(五四)?青年|(六一)?儿童|(七一)?建党|(八一)?建军|教师|情人|愚人|万圣|护士)节|'\
    r'地球日|三[\.•·・]?一五|双(十一|11)|(.{1,4})?消费者权益日)'
# 固定农历节日
FIXED_LUNAR_FESTIVAL = r'((春|填仓|上巳|寒食|浴佛|姑姑|财神|下元|寒衣)节|'\
    r'(龙抬头|除夕)|' \
    r'(大年初[一二三四五六七八九十])|'\
    r'(端午|端阳|七夕|元宵|中秋|重阳|腊八|中元)(节)?)'
# 规律公历节日
REGULAR_FOREIGN_FESTIVAL = r'(感恩|母亲|父亲)节'

# time_delta 数字正则
DELTA_NUM_STRING = r'(([一两二三四五六七八九十百千万零]+点)?[一两二三四五六七八九十百千万零]+|([\d十百千万,]+\.)?[\d十百千万,]+)'
QUARTER_NUM_STRING = r'[一两二三四1-4]'

# 单个数字正则
SINGLE_NUM_STRING = r'[一两二三四五六七八九十\d]'

# time_delta 正则
YEAR_DELTA_STRING = ''.join([DELTA_NUM_STRING, r'[多余]?(周)?年(多)?', I, '半年', I, SINGLE_NUM_STRING, '年半'])
SOLAR_SEASON_DELTA_STRING = ''.join([DELTA_NUM_STRING, r'个(多)?季度'])
MONTH_DELTA_STRING = ''.join([DELTA_NUM_STRING, r'(多)?个(多)?月', I, '(俩|三|仨)月', I, '半(个(多)?)?月', I,
                              SINGLE_NUM_STRING, '个半月'])
WORKDAY_DELTA_STRING = ''.join([DELTA_NUM_STRING, r'[多余]?(个)?(工作|交易)日'])
DAY_DELTA_STRING = ''.join([DELTA_NUM_STRING, r'[多余]?[天日]', I, '半天', I, SINGLE_NUM_STRING, '天半'])
WEEK_DELTA_STRING = ''.join([DELTA_NUM_STRING, r'[多余]?((个(多)?)?(星期|礼拜)|周(?!年))', I, r'俩(星期|礼拜)'])
HOUR_DELTA_STRING = ''.join([DELTA_NUM_STRING, r'[多余]?(个(多)?)?(小时|钟头)', I,
                             '半(个(多)?)?(小时|钟头)', I, '俩(小时|钟头)', I, SINGLE_NUM_STRING, '个半(小时|钟头)'])
QUARTER_DELTA_STRING = ''.join([QUARTER_NUM_STRING, '刻钟'])
MINUTE_DELTA_STRING = ''.join([DELTA_NUM_STRING, r'[多余]?分(钟)?(半)?', I, '半分钟', I,
                               SINGLE_NUM_STRING, '+分半(钟)?'])
SECOND_DELTA_STRING = ''.join([DELTA_NUM_STRING, r'[多余]?秒(钟)?'])


# 将时间进行转换
# DELTA_SUB = r'([之以]?[内前后上下来])'
DELTA_SUB = r'([之以]?[内前后来])'

########################################################################
# 时间 NER 字符规则
TIME_CHAR_STRING = ''.join(
    [r'(现在|开始|黎明|过去|愚人|感恩|圣诞|儿童|劳动|父亲|母亲|礼拜|霜降|立春|立冬|小寒|大寒|',
     r'立夏|立秋|冬至|',
     r'[102年月日3589647时午至天上个分今下\:\-点晚前一小后周起内以底三晨钟来半两凌当十份季去早多第五中初\.度二从六期旬到间四节号：',
     r'代\~\—～春明昨星末秋之同·世纪本七九秒每次八夏/夜零正冬腊余工作元国清傍交易首 ()（）、万宵全暑头端庆旦－际消费者权益大里农阴历双财',
     r'近运深, ”夕〇几汛假壹无数白百刻许左右的这本])+'])
FAKE_POSITIVE_START_STRING = r'[起到至以开－\—\-\~～]'  # 此字符串不可作为时间串的开始， `以来|开始` 为取首字
FAKE_POSITIVE_END_STRING = r'[到至－\—\-\~～]'  # 此字符串不可作为时间串的结束

########################################################################
# 货币金额 NER 字符规则
MONEY_PREFIX_STRING = r'((将)?近|只有|仅|(大)?约(莫|合)?|大概|至少(要)?|不(到|足|超过)?|逾|高于|(高)?达(到)?|^上|(超)?过|超)'
MONEY_SUFFIX_STRING = r'(以上|以下|左右|上下)'

MONEY_BLUR_STRING = r'((大)?约(莫|合)?|大概|左右|上下)'
MONEY_MINUS_STRING = r'((将)?近|不(到|足|超过)?|以下)'
MONEY_PLUS_STRING = r'(至少(要)?|逾|高于|上|(超)?过|超|以上)'

MONEY_NUM_MIDDLE_STRING = r'[,， ]'
# 用于检测字符串是否有误，直接报错
MONEY_NUM_STRING = r'^[ \.多个数几百佰k千仟w万萬亿十拾兆〇O0-9零０-９一二两三四五六七八九壹贰俩叁弎仨肆伍陆柒捌玖]+$'

MONEY_KUAI_MAO_JIAO_FEN_STRING = r'[分角毛块]'
MONEY_PREFIX_CASE_STRING = r'(港币|人民币|(新)?台币)'

# 货币单位，该货币单位也包含了上述 MONEY_PREFIX_CASE_STRING
# 雷亚尔：巴西货币
MONEY_SUFFIX_CASE_STRING = r'((分|角|毛|块|元)钱?|(人民|港|日|澳|(新)?台)币|圆(整)?|英镑|美(金|分|刀)|马克|法郎|卢布|泰铢|'\
                           r'元((人民|港|日|澳|韩|(新)?台)币)?|(美|港|澳门|日|韩|缅|马|新加坡|欧|加|加拿大|新西兰|澳|澳大利亚)元|'\
                           r'(越(南)?)盾|雷亚尔)'

# MONEY_SPAN_GAP_START = r'(从)'
# MONEY_SPAN_GAP_MIDDLE = r'(\~+|\-+|～+|－+|至(?!少)|(?<![达不])到)'

# 该金额抽取正则将会包含一些错例，例如“2019-07-18”此类字符串也将被抽取出，即范围覆盖面过大，且还能被正确解析，因此需要进行错例删除正则进行处理
MONEY_CHAR_STRING = r'((将)?近|只有|仅|(大)?约(莫|合)?|大概|至少(要)?|不(到|足|超过)?|逾|高于|(高)?达(到)?|^上|(超)?过|超|' \
                    r'以上|以下|左右|上下|港币|人民币|(新)?台币|(分|角|毛|块|元)钱?|(人民|港|日|澳|(新)?台)币|圆(整)?|英镑|' \
                    r'美(金|分|刀)|马克|法郎|卢布|泰铢|元((人民|港|日|澳|韩|(新)?台)币)?|(美|港|澳门|日|韩|缅|马|新加坡|欧|' \
                    r'加|加拿大|新西兰|澳|澳大利亚)元|(越(南)?)盾|雷亚尔|' \
                    r'[分角毛块 \.\,\-\~—－～，余多个数几百佰k千仟w万萬亿十拾兆〇O0-9零０-９一二两三四五六七八九壹贰俩叁弎仨肆伍陆柒捌玖\(\)（）不含])+'

# MONEY_EXCEPTION_STRING = r'(\d+[\-\~—－～]\d+[\-\~—－～]\d+|)'


from abc import abstractclassmethod

from datasetstation.config import AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, S3_BUCKET_URI

from datasets.filesystems import S3FileSystem


class StorageBackend(object):

    backend_storage = None
    backend_uri = None

    @abstractclassmethod
    def download():
        pass


class S3StorageBackend(S3FileSystem, StorageBackend):

    backend_uri = S3_BUCKET_URI

    def __init__(self):
        super(S3StorageBackend, self).__init__(
            key=AWS_ACCESS_KEY_ID,
            secret=AWS_SECRET_ACCESS_KEY,
            client_kwargs={"region_name": "cn-north-1"},
        )


# coding:utf8


import re
import collections
import configparser

from fastweb.util.log import recorder
from fastweb.util.python import to_iter, str2everything
from fastweb.exception import ParameterError


class ConfigurationParser(object):
    """Configuration Parser"""

    parser = {'ini': '_ini_parser'}

    def __init__(self, t, **setting):
        """parse config file

        :parameter:
          - `t`: type
          - `setting`: setting

        parse result:
            `configs`: {'section': {setting}}
        """

        self.configs = None
        self.get_configs(t, setting)

    @staticmethod
    def _check_setting(eattr, setting):
        """check backend setting

        :parameter:
          - `eattr`: essential attribute
          - `setting`: setting
        """

        for attr in eattr:
            v = setting.get(attr)
            if not v:
                recorder('CRITICAL', 'configuration backend setting error! '
                                     'right options {options}'.format(options=eattr))
                raise ParameterError

    def _ini_parser(self, setting):
        """ini file parser

        :parameter:
          - `setting`: setting
        """

        eattr = ['path']
        self._check_setting(eattr, setting)
        path = setting.get('path')
        cf = configparser.ConfigParser()
        cf.read(path)
        configs = collections.defaultdict(dict)

        for section in cf.sections():
            options = cf.items(section)
            for key, value in options:
                value = str2everything(value)
                configs[section][key] = value
        return configs

    def get_configs(self, t, setting):
        """get configs from the type

        :parameter:
          - `backend`: type
          - `setting`: setting
        """

        parser = self.parser.get(t)
        if parser:
            self.configs = getattr(self, parser)(setting)
        else:
            raise ParameterError

    def get_components(self, components):
        """get components protocol

        :parameter:
          - `components`: component name, may be a list

        :return:
          { section:
            { 'component': component_name,
             'object': component_type }
          }
        """

        match_components = {}
        components = to_iter(components)

        for component in components:
            component_exp = r'(%s):(\w*)' % component
            exp = re.compile(component_exp)

            for section in list(self.configs.keys()):
                match = exp.match(section)
                if match:
                    match_components[section] = {'component': match.group(1), 'object': match.group(2)}

        return match_components



import re
import string

from ..rule_pattern import *
from ..hanzi import punctuation


__all__ = ["Extractor"]


class Extractor(object):
    """规则抽取器"""

    def __init__(self):
        self.money_pattern = None
        self.email_pattern = None
        self.email_domain_pattern = None
        self.email_prefix_pattern = None
        self.url_pattern = None
        self.phone_number_pattern = None
        self.ip_address_pattern = None
        self.id_card_pattern = None
        self.html_tag_pattern = None
        self.qq_pattern = None
        self.strict_qq_pattern = None
        self.wechat_id_pattern = None
        self.strict_wechat_id_pattern = None
        self.cell_phone_pattern = None
        self.landline_phone_pattern = None
        self.phone_prefix_pattern = None
        self.extract_parentheses_pattern = None
        self.remove_parentheses_pattern = None
        self.parentheses_pattern = PARENTHESES_PATTERN
        self.parentheses_dict = None
        self.redundant_pattern = None
        self.exception_pattern = None
        self.full_angle_pattern = None
        self.chinese_char_pattern = None
        self.chinese_chars_pattern = None

    @staticmethod
    def _extract_base(pattern, text, with_offset=False):
        """正则抽取器的基础函数
        Args:
            pattern(re.compile): 正则表达式对象
            text(str): 字符串文本
            with_offset(bool): 是否携带 offset （抽取内容字段在文本中的位置信息）
        Returns:
            list: 返回结果
        """
        if with_offset:
            results = [
                {
                    "text": item.group(1),
                    "offset": (item.span()[0] - 1, item.span()[1] - 1),
                }
                for item in pattern.finditer(text)
            ]
        else:
            results = [item.group(1) for item in pattern.finditer(text)]

        return results

    def remove_redundant_char(self, text, redundant_chars=None):
        """去除冗余字符
        Args:
            text(str): 待处理文本
            redundant_chars(str|list): 自定义待去除的冗余字符串 或 list，
                如 ”哈嗯~“，或 ['哈', '嗯', '\u3000']，若不指定则采用默认的冗余字符串。
        Returns:
            删除冗余字符后的文本
        """
        if self.redundant_pattern is None:
            pattern_list = list()
            if redundant_chars is None:
                redundant_chars = REDUNDANT_PATTERN

            for char in redundant_chars:
                pattern_tmp = "(?<={char}){char}+".format(char=re.escape(char))
                pattern_list.append(pattern_tmp)

            redundant_pattern = "|".join(pattern_list)
            self.redundant_pattern = re.compile(redundant_pattern)

        return self.redundant_pattern.sub("", text)

    def clean_text(
        self,
        text,
        remove_html_tag=True,
        convert_full2half=True,
        remove_exception_char=True,
        remove_url=True,
        remove_redundant_char=True,
        remove_parentheses=True,
        remove_email=True,
        remove_phone_number=True,
        remove_punctuation=True,
        remove_whitespace=True,
        remove_alphabet=False,
        delete_prefix=False,
        redundant_chars=None,
    ):
        """清洗文本，关键字参数均默认为 True
        Args:
            text(str): 待清理文本
            remove_html_tag(bool): 是否删除html标签，如 <span> 等
            remove_exception_char(bool): 是否删除异常字符，如“敩衞趑”等
            convert_full2half(bool): 是否将全角字符转换为半角
            remove_redundant_char(bool): 是否删除冗余字符，如“\n\n\n”，修剪为“\n”
            remove_parentheses(bool): 是否删除括号及括号内内容，如“（记者：小丽）”
            remove_url(bool): 是否删除 url 链接
            remove_email(bool): 是否删除 email
            remove_phone_number(bool): 是否删除电话号码
            delete_prefix(bool): 是否删除 email 和 电话号码的前缀，如 `E-mail: xxxx@gmail.com`
            redundant_chars(str|list|None): 自定义待去除的冗余字符串 或 list，
                如 ”哈嗯~“，或 ['哈', '嗯', '\u3000']，若不指定则采用默认的冗余字符串。
        Returns:
            str: 清理后的文本
        """

        if remove_html_tag:
            text = self.remove_html_tag(text)
        if remove_exception_char:
            text = self.remove_exception_char(text)
        if convert_full2half:
            text = self.convert_full2half(text)
        if remove_redundant_char:
            text = self.remove_redundant_char(text, redundant_chars=redundant_chars)
        if remove_parentheses:
            text = self.remove_parentheses(text)
        if remove_url:
            text = self.remove_url(text)
        if remove_email:
            text = self.remove_email(text, delete_prefix=delete_prefix)
        if remove_phone_number:
            text = self.remove_phone_number(text, delete_prefix=delete_prefix)
        if remove_punctuation:
            text = self.remove_punctuation(text)
        if remove_whitespace:
            text = self.remove_whitespace(text)
        if remove_alphabet:
            text = self.remove_alphabet(text)

        return text

    def convert_full2half(self, text):
        """将全角字符转换为半角字符
        其中分为空格字符和非空格字符
        """
        if self.full_angle_pattern is None:
            self.full_angle_pattern = str.maketrans(
                FULL_ANGLE_ALPHABET, HALF_ANGLE_ALPHABET
            )

        return text.translate(self.full_angle_pattern)

    def extract_email(self, text, detail=False):
        """提取文本中的 E-mail
        Args:
            text(str): 字符串文本
            detail(bool): 是否携带 offset (E-mail 在文本中的位置信息）
        Returns:
            list: email列表
        """
        if self.email_pattern is None:
            self.email_pattern = re.compile(EMAIL_PATTERN)

        text = "".join(["#", text, "#"])
        results = self._extract_base(self.email_pattern, text, with_offset=detail)
        if not detail:
            return results
        else:
            if self.email_domain_pattern is None:
                self.email_domain_pattern = re.compile(EMAIL_DOMAIN_PATTERN)

            detail_results = list()
            for item in results:
                domain_name = self.email_domain_pattern.search(item["text"]).group(1)  # type: ignore
                item.update({"domain_name": domain_name})  # type: ignore
                detail_results.append(item)
            return detail_results

    def extract_id_card(self, text, detail=False):
        """提取文本中的 ID 身份证号
        Args:
            text(str): 字符串文本
            detail(bool): 是否携带 offset （身份证在文本中的位置信息）
        Returns:
            list: 身份证信息列表
        """
        if self.id_card_pattern is None:
            self.id_card_pattern = re.compile(ID_CARD_PATTERN)

        text = "".join(["#", text, "#"])
        return self._extract_base(self.id_card_pattern, text, with_offset=detail)

    def extract_ip_address(self, text, detail=False):
        """提取文本中的 IP 地址
        Args:
            text(str): 字符串文本
            detail(bool): 是否携带 offset (IP 地址在文本中的位置信息）
        Returns:
            list: IP 地址列表
        """
        if self.ip_address_pattern is None:
            self.ip_address_pattern = re.compile(IP_ADDRESS_PATTERN)

        text = "".join(["#", text, "#"])
        return self._extract_base(self.ip_address_pattern, text, with_offset=detail)

    def extract_money(self, text, detail=False):
        """从文本中抽取出金额字符串，可以和 money_standardization 函数配合使用，
        得到数字金额。(TO BE DEPRECATED.)
        Args:
            text(str): 字符串文本
            detail(bool): 返回字符串的详细信息 offset，默认为 False
        Returns:
            list: 货币金额列表
        Examples:
            >>> import jionlp as jio
            >>> money_result = jio.extract_money(
                    '张三赔偿李四人民币车费601,293.11元，工厂费一万二千三百四十五元,利息9佰日元，打印费十块钱。')
            >>> print(money_result)
            # ['601,293.11元', '一万二千三百四十五元', '9佰日元', '十块钱']
        """
        if self.money_pattern is None:
            self.money_pattern = re.compile(MONEY_PATTERN)

        money_result = list()
        for item in self.money_pattern.finditer(text):
            if detail:
                money_result.append({"text": item.group(), "offset": list(item.span())})
            else:
                money_result.append(item.group())

        return money_result

    def extract_phone_number(self, text, detail=False):
        """从文本中抽取出电话号码
        Args:
            text(str): 字符串文本
            detail(bool): 是否携带 offset （电话号码在文本中的位置信息）
        Returns:
            list: 电话号码列表
        """
        if self.cell_phone_pattern is None:
            self.cell_phone_pattern = re.compile(CELL_PHONE_PATTERN)

        if self.landline_phone_pattern is None:
            self.landline_phone_pattern = re.compile(LANDLINE_PHONE_PATTERN)

        text = "".join(["#", text, "#"])
        cell_results = self._extract_base(
            self.cell_phone_pattern, text, with_offset=detail
        )
        landline_results = self._extract_base(
            self.landline_phone_pattern, text, with_offset=detail
        )

        if not detail:
            return cell_results + landline_results
        else:
            detail_results = list()
            for item in cell_results:
                item.update({"type": "cell_phone"})  # type: ignore
                detail_results.append(item)
            for item in landline_results:
                item.update({"type": "landline_phone"})  # type: ignore
                detail_results.append(item)
            return detail_results

    def extract_qq(self, text, detail=False, strict=True):
        """从文本中抽取出 QQ 号码
        Args:
            text(str): 字符串文本
            detail(bool): 是否携带 offset （QQ 在文本中的位置信息）
            strict(bool): QQ号很容易和其他数字混淆，因此选择采用严格或宽松规则匹配
        Returns:
            list: QQ 号列表
        """
        if self.qq_pattern is None:
            self.qq_pattern = re.compile(QQ_PATTERN)
            self.strict_qq_pattern = re.compile(STRICT_QQ_PATTERN)

        text = "".join(["#", text, "#"])
        tmp_res = self._extract_base(self.qq_pattern, text, with_offset=detail)

        if not strict:
            return tmp_res
        else:
            # 将无法匹配 qq 字符的 qq 号删除
            match_flag = self.strict_qq_pattern.search(text)  # type: ignore
            if match_flag:
                return tmp_res
            else:
                return list()

    def extract_wechat_id(self, text, detail=False, strict=True):
        """从文本中抽取出 微信号 号码
        微信官方定义的微信号规则：
        1、可使用6-20个字母、数字、下划线和减号
        2、必须以字母开头(字母不区分大小写)
        3、不支持设置中文。
        Args:
            text(str): 字符串文本
            detail(bool): 是否携带 offset （微信号 在文本中的位置信息）
            strict(bool): 微信号 很容易和其他数字混淆，因此选择采用严格或宽松规则匹配
        Returns:
            list: 微信号 列表
        """
        if self.wechat_id_pattern is None:
            self.wechat_id_pattern = re.compile(WECHAT_ID_PATTERN)
            self.strict_wechat_id_pattern = re.compile(STRICT_WECHAT_ID_PATTERN)

        text = "".join(["#", text, "#"])
        tmp_res = self._extract_base(self.wechat_id_pattern, text, with_offset=True)

        if not strict:
            return tmp_res
        else:
            # 将无法匹配 微信号 字符的 微信号 删除
            # 注意该规则未经充分的数据验证，仅凭启发式规则定义而成。
            final_res = list()
            for item in tmp_res:
                end_offset = item["offset"][0]  # type: ignore
                # 此处的考察范围 8 为一个默认值
                start_offset = max(0, end_offset - 8)  # type: ignore
                match_flag = self.strict_wechat_id_pattern.search(  # type: ignore
                    text[start_offset:end_offset]
                )

                if match_flag:
                    if detail:
                        final_res.append(item)
                    else:
                        final_res.append(item["text"])  # type: ignore

            return final_res

    def extract_url(self, text, detail=False):
        """提取文本中的url链接
        Args:
            text(str): 字符串文本
            detail(bool): 是否携带 offset （URL 在文本中的位置信息）
        Returns:
            list: url列表
        """
        if self.url_pattern is None:
            self.url_pattern = re.compile(URL_PATTERN)
        text = "".join(["￥", text, "￥"])  # 因 # 可出现于 url

        return self._extract_base(self.url_pattern, text, with_offset=detail)

    """
    def _extract_parentheses(self, text, parentheses=PARENTHESES_PATTERN):
        # 额外分支 Ghs 提供的方法
        if self.extract_parentheses_pattern is None or self.parentheses_pattern != parentheses:
            import regex as reg
            self.parentheses_pattern = parentheses
            parentheses_per = zip(self.parentheses_pattern[:-1], self.parentheses_pattern[1:])
            self.extract_parentheses_pattern = f"(?:{'|'.join('{left}([^{left}{right}]*){right}'.format(left=reg.escape(f), right=reg.escape(e)) for f, e in parentheses_per)})"
        return [{'context': [j for j in i.groups() if j][0], 'offset': i.span(), 'origin': i.group()}
                for i in reg.compile(self.extract_parentheses_pattern).finditer(text)]
    """

    def extract_parentheses(self, text, parentheses=PARENTHESES_PATTERN, detail=False):
        """提取文本中的括号及括号内内容，当有括号嵌套时，提取每一对
        成对的括号的内容
        Args:
            text(str): 字符串文本
            parentheses: 要删除的括号类型，格式为:
                '左括号1右括号1左括号2右括号2...'，必须为成对的括号如'{}()[]'，
                默认为self.parentheses
            detail: 是否打印括号内容位置信息
        Returns:
            list: [
                    {
                        'context'(str): 'the context between parentheses',
                        'offset'(tuple): 'the location of extracted text'
                    },  # 当 detail 为 True 时
                    'the context between parentheses',  # 当 detail 为 False 时
                    ...
                ]
        """
        if (
            self.extract_parentheses_pattern is None
            or self.parentheses_pattern != parentheses
        ):
            self.parentheses_pattern = parentheses

            extract_pattern = "[" + re.escape(self.parentheses_pattern) + "]"
            extract_pattern = re.compile(extract_pattern)

            p_length = len(self.parentheses_pattern)

            parentheses_dict = dict()
            for i in range(0, p_length, 2):
                value = self.parentheses_pattern[i]
                key = self.parentheses_pattern[i + 1]
                parentheses_dict.update({key: value})

            self.parentheses_dict = parentheses_dict
            self.extract_parentheses_pattern = extract_pattern

        content_list = list()
        parentheses_list = list()
        idx_list = list()
        finditer = self.extract_parentheses_pattern.finditer(text)
        for i in finditer:
            idx = i.start()
            parentheses = text[idx]

            if parentheses in self.parentheses_dict.keys():  # type: ignore
                if len(parentheses_list) > 0:
                    if parentheses_list[-1] == self.parentheses_dict[parentheses]:  # type: ignore
                        parentheses_list.pop()
                        if detail:
                            start_idx = idx_list.pop()
                            end_idx = idx + 1
                            content_list.append(
                                {
                                    "content": text[start_idx:end_idx],
                                    "offset": (start_idx, end_idx),
                                }
                            )
                        else:
                            content_list.append(text[idx_list.pop() : idx + 1])
            else:
                parentheses_list.append(parentheses)
                idx_list.append(idx)

        return content_list

    def remove_email(self, text, delete_prefix=False):
        """删除文本中的 email
        Args:
            text(str): 字符串文本
            delete_prefix(bool): 删除电子邮箱前的前缀符，如 `E-mail: xxxx@163.com`
                由于计算前缀符的匹配，该方法计算效率会慢。
        Returns:
            str: 删除 email 后的文本
        """
        if self.email_pattern is None:
            self.email_pattern = re.compile(EMAIL_PATTERN)
            self.email_prefix_pattern = re.compile(EMAIL_PREFIX_PATTERN)

        text = "".join(["#", text, "#"])
        if not delete_prefix:
            text = self.email_pattern.sub("", text)
            return text[1:-1]
        else:

            results = self._extract_base(self.email_pattern, text, with_offset=True)
            prefix_results = self._extract_base(
                self.email_prefix_pattern, text, with_offset=True
            )

            offset_list = [item["offset"][0] for item in results]  # type: ignore

            clean_prefix_offsets = [
                item["offset"]  # type: ignore
                for item in prefix_results
                if item["offset"][1] in offset_list  # type: ignore
            ]

            final_text_list = list()
            for idx, item in enumerate(clean_prefix_offsets):
                if idx == 0:
                    final_text_list.append(text[0 : item[0] + 1])  # type: ignore

                if idx == len(clean_prefix_offsets) - 1:
                    final_text_list.append(text[item[1] + 1 :])  # type: ignore
                else:
                    final_text_list.append(
                        text[item[1] + 1 : clean_prefix_offsets[idx + 1][0] + 1]  # type: ignore
                    )

            text = "".join(final_text_list)
            text = self.email_pattern.sub("", text)

        return text[1:-1]

    def remove_exception_char(self, text):
        """删除文本中的异常字符
        Args:
            text(str): 字符串文本
        Returns:
             str: 删除异常字符后的文本
        """
        if self.exception_pattern is None:
            self.exception_pattern = re.compile(EXCEPTION_PATTERN)

        return self.exception_pattern.sub(" ", text)

    def remove_html_tag(self, text):
        """删除文本中的 html 标签
        Args:
            text(str): 字符串文本
        Returns:
            str: 删除 html 标签后的文本
        """
        if self.html_tag_pattern is None:
            self.html_tag_pattern = re.compile(HTML_TAG_PATTERN)
        return re.sub(self.html_tag_pattern, "", text)

    def remove_id_card(self, text):
        """删除文本中的身份证号
        Args:
            text(str): 字符串文本
        Returns:
            str: 删除身份证 id 后的文本
        """
        if self.id_card_pattern is None:
            self.id_card_pattern = re.compile(ID_CARD_PATTERN)

        text = "".join(["#", text, "#"])
        return self.id_card_pattern.sub("", text)[1:-1]

    def remove_ip_address(self, text):
        """删除文本中的 ip 地址
        Args:
            text(str): 字符串文本
        Returns:
            str: 删除 ip 地址后的文本
        """
        if self.ip_address_pattern is None:
            self.ip_address_pattern = re.compile(IP_ADDRESS_PATTERN)

        text = "".join(["#", text, "#"])
        return self.ip_address_pattern.sub("", text)[1:-1]

    def remove_parentheses(self, text, parentheses=PARENTHESES_PATTERN):
        """删除文本中的括号及括号内内容
        Args:
            text(str): 字符串文本
            parentheses: 要删除的括号类型，格式为:
                '左括号1右括号1左括号2右括号2...'，必须为成对的括号如'{}()[]'，
                默认为self.parentheses
        Returns:
            str: 删除括号及括号中内容后的文本
        """
        if (
            self.remove_parentheses_pattern is None
            or self.parentheses_pattern != parentheses
        ):
            self.parentheses_pattern = parentheses

            p_length = len(self.parentheses_pattern)
            remove_pattern_list = list()
            remove_pattern_format = "{left}[^{left}{right}]*{right}"

            for i in range(0, p_length, 2):
                left = re.escape(self.parentheses_pattern[i])
                right = re.escape(self.parentheses_pattern[i + 1])
                remove_pattern_list.append(
                    remove_pattern_format.format(left=left, right=right)
                )

            remove_pattern = "|".join(remove_pattern_list)
            remove_pattern = re.compile(remove_pattern)

            self.remove_parentheses_pattern = remove_pattern

        length = len(text)
        while True:
            text = self.remove_parentheses_pattern.sub("", text)
            if len(text) == length:
                return text
            length = len(text)

    def remove_phone_number(self, text, delete_prefix=False):
        """删除文本中的电话号码
        Args:
            text(str): 字符串文本
            delete_prefix(bool): 删除电话号码前缀，如 `电  话：198xxxxxxxx`
        Returns:
            str: 删除电话号码后的文本
        """
        if self.cell_phone_pattern is None:
            self.cell_phone_pattern = re.compile(CELL_PHONE_PATTERN)
            self.phone_prefix_pattern = re.compile(PHONE_PREFIX_PATTERN)

        if self.landline_phone_pattern is None:
            self.landline_phone_pattern = re.compile(LANDLINE_PHONE_PATTERN)
            self.phone_prefix_pattern = re.compile(PHONE_PREFIX_PATTERN)

        text = "".join(["#", text, "#"])

        if not delete_prefix:
            text = self.cell_phone_pattern.sub("", text)
            text = self.landline_phone_pattern.sub("", text)

        else:
            cell_results = self._extract_base(
                self.cell_phone_pattern, text, with_offset=True
            )
            landline_results = self._extract_base(
                self.landline_phone_pattern, text, with_offset=True
            )
            results = sorted(
                cell_results + landline_results, key=lambda i: i["offset"][0]  # type: ignore
            )

            prefix_results = self._extract_base(
                self.phone_prefix_pattern, text, with_offset=True
            )

            offset_list = [item["offset"][0] for item in results]  # type: ignore

            clean_prefix_offsets = [
                item["offset"]  # type: ignore
                for item in prefix_results
                if item["offset"][1] in offset_list  # type: ignore
            ]

            final_text_list = list()
            for idx, item in enumerate(clean_prefix_offsets):
                if idx == 0:
                    final_text_list.append(text[0 : item[0] + 1])  # type: ignore

                if idx == len(clean_prefix_offsets) - 1:
                    final_text_list.append(text[item[1] + 1 :])  # type: ignore
                else:
                    final_text_list.append(
                        text[item[1] + 1 : clean_prefix_offsets[idx + 1][0] + 1]  # type: ignore
                    )

            text = "".join(final_text_list)
            text = self.cell_phone_pattern.sub("", text)
            text = self.landline_phone_pattern.sub("", text)

        return text[1:-1]

    def remove_qq(self, text, strict=True):
        """删除文本中的电 QQ 号
        Args:
            text(str): 字符串文本
            strict(bool): QQ 号容易与其他数字混淆，因此选择严格规则或宽松规则
        Returns:
            str: 删除 QQ 后的文本
        """
        if self.qq_pattern is None:
            self.qq_pattern = re.compile(QQ_PATTERN)
            self.strict_qq_pattern = re.compile(STRICT_QQ_PATTERN)

        if strict:
            # 将无法匹配 qq 字符的文本直接返回
            match_flag = self.strict_qq_pattern.search(text)  # type: ignore
            if not match_flag:
                return text

        text = "".join(["#", text, "#"])
        return self.qq_pattern.sub("", text)[1:-1]

    def remove_url(self, text):
        """删除文本中的 url 链接
        Args:
            text(str): 字符串文本
        Returns:
            text: 删除 url 链接后的文本
        """
        if self.url_pattern is None:
            self.url_pattern = re.compile(URL_PATTERN)

        text = "".join(["￥", text, "￥"])
        return self.url_pattern.sub("", text)[1:-1]

    def replace_email(self, text, token="<email>"):
        """替换文本中的 email 为归一化标签
        Args:
            text(str): 字符串文本
            token(str): 替换 email 的 token，默认为 `<email>`，与预训练模型保持一致
        Returns:
            str: 替换 email 为归一化 token 后的文本
        """
        if self.email_pattern is None:
            self.email_pattern = re.compile(EMAIL_PATTERN)
            self.email_prefix_pattern = re.compile(EMAIL_PREFIX_PATTERN)

        text = "".join(["#", text, "#"])

        text = self.email_pattern.sub(token, text)
        return text[1:-1]

    def replace_id_card(self, text, token="<id>"):
        """替换文本中的身份证号为归一化标签
        Args:
            text(str): 字符串文本
            token(str): 替换 id 的 token，默认为 `<id>`，与预训练模型保持一致
        Returns:
            str: 替换身份证 id 为归一化 token 后的文本
        """
        if self.id_card_pattern is None:
            self.id_card_pattern = re.compile(ID_CARD_PATTERN)

        text = "".join(["#", text, "#"])
        return self.id_card_pattern.sub(token, text)[1:-1]

    def replace_ip_address(self, text, token="<ip>"):
        """替换文本中的 ip 地址为归一化标签
        Args:
            text(str): 字符串文本
            token(str): 替换 ip 的 token，默认为 `<ip>`，与预训练模型保持一致
        Returns:
            str: 替换 ip 地址为归一化 token 后的文本
        """
        if self.ip_address_pattern is None:
            self.ip_address_pattern = re.compile(IP_ADDRESS_PATTERN)

        text = "".join(["#", text, "#"])
        return self.ip_address_pattern.sub(token, text)[1:-1]

    def replace_phone_number(self, text, token="<tel>"):
        """替换文本中的电话号码为归一化标签 token
        Args:
            text(str): 字符串文本
            token(str): 替换 tel 的 token，默认为 `<tel>`，与预训练模型保持一致
        Returns:
            str: 替换电话号码后为归一化标签 token 的文本
        """
        if self.cell_phone_pattern is None:
            self.cell_phone_pattern = re.compile(CELL_PHONE_PATTERN)
            self.phone_prefix_pattern = re.compile(PHONE_PREFIX_PATTERN)

        if self.landline_phone_pattern is None:
            self.landline_phone_pattern = re.compile(LANDLINE_PHONE_PATTERN)
            self.phone_prefix_pattern = re.compile(PHONE_PREFIX_PATTERN)

        text = "".join(["#", text, "#"])

        text = self.cell_phone_pattern.sub(token, text)
        text = self.landline_phone_pattern.sub(token, text)

        return text[1:-1]

    def replace_qq(self, text, strict=True, token="<qq>"):
        """替换文本中的电 QQ 号为归一化标签
        Args:
            text(str): 字符串文本
            strict(bool): QQ 号容易与其他数字混淆，因此选择严格规则或宽松规则
            token(str): 替换 QQ 的 token，默认为 `<url>`，与预训练模型保持一致
        Returns:
            str: 替换 QQ 为归一化 token 后的文本
        """
        if self.qq_pattern is None:
            self.qq_pattern = re.compile(QQ_PATTERN)
            self.strict_qq_pattern = re.compile(STRICT_QQ_PATTERN)

        if strict:
            # 将无法匹配 qq 字符的文本直接返回
            match_flag = self.strict_qq_pattern.search(text)  # type: ignore
            if not match_flag:
                return text

        text = "".join(["#", text, "#"])
        return self.qq_pattern.sub(token, text)[1:-1]

    def replace_url(self, text, token="<url>"):
        """将文本中的 url 链接归一化
        Args:
            text(str): 字符串文本
            token(str): 替换 url 的 token，默认为 `<url>`，与预训练模型保持一致
        Returns:
            text: 将 url 链接文本统一替换成标准字符串，默认为 token `<url>`
                token可以自行定义。
        """
        if self.url_pattern is None:
            self.url_pattern = re.compile(URL_PATTERN)

        text = "".join(["￥", text, "￥"])
        return self.url_pattern.sub(token, text)[1:-1]

    def replace_chinese(self, text, substitute=r" "):
        """删除文本中的所有中文字符串
        Args:
            text(str): 输入的文本
            substitute(str): 将中文文字，替换为何种字符串，默认为一个空格
        Return:
            list(str): 中文文本列表，若两段中文之间有其它字符，则按序排列在列表中
        Examples:
            >>> import jionlp as jio
            >>> print(jio.replace_chinese('【新华社消息】（北京时间）从昨天...'))
            # '【     】（    ）   ...'
        """
        if text == "":
            return list()

        if self.chinese_char_pattern is None:
            self.chinese_char_pattern = re.compile(CHINESE_CHAR_PATTERN)

        if type(substitute) is not str:
            raise TypeError("the `substitute` is not string.")

        text_without_chinese = self.chinese_char_pattern.sub(substitute, text)

        return text_without_chinese

    def extract_chinese(self, text):
        """抽取出文本中的所有中文字符串
        Args:
            text(str): 输入的文本
        Return:
            list(str): 中文文本列表，若两段中文之间有其它字符，则按序排列在列表中
        Examples:
            >>> import jionlp as jio
            >>> print(jio.extract_chinese('【新华社消息】（北京时间）从昨天...'))
            # ['新华社消息', '北京时间','从昨天']
        """
        if text == "":
            return list()

        if self.chinese_chars_pattern is None:
            self.chinese_chars_pattern = re.compile(CHINESE_CHAR_PATTERN + "+")

        chinese_text_list = self.chinese_chars_pattern.findall(text)

        return chinese_text_list

    def check_chinese_char(self, text):
        """检查文本中是否包含中文字符
        Args:
            text(str): 输入的文本
        Return:
            bool: 文本中是否包含中文字符
        Examples:
            >>> import jionlp as jio
            >>> print(jio.check_chinese_char('【新华社消息】（北京时间）从昨天...'))
            # True
        """
        if text == "":
            return False

        if self.chinese_char_pattern is None:
            self.chinese_char_pattern = re.compile(CHINESE_CHAR_PATTERN)

        if self.chinese_char_pattern.search(text):
            return True

        return False

    def remove_punctuation(self, text):
        """移除中英文中的标点符号"""
        return text.translate(str.maketrans('', '', string.punctuation + punctuation))

    def remove_whitespace(self, text):
        return text.translate(str.maketrans('', '', ' \n\t\r'))

    def remove_alphabet(self, text):
        return text.translate(str.maketrans('', '', string.ascii_lowercase + string.ascii_uppercase))


# coding:utf8

import re
import inspect


def get_class_from_frame(fr):
    args, _, _, value_dict = inspect.getargvalues(fr)

    if len(args) and args[0] == 'self':
        instance = value_dict.get('self', None)

        if instance:
            return getattr(instance, '__class__', None)

    return None


def get_file_name_in_full_path(file_path):
    return file_path.split('/')[-1]


def get_meta_data():
    frames = inspect.stack()
    chain_list = []

    for i in range(0, len(frames)):
        _, file_path, _, func_name, _, _ = frames[i]
        file_name = get_file_name_in_full_path(file_path)

        try:
            args = re.findall('\((.*)\)', frames[i + 1][-2][0])[0]
        except IndexError as e:
            func = get_class_from_frame(frames[2][0])

            if func:
                func_name = func.__name__

            args = ''
        except TypeError as e:
            func = get_class_from_frame(frames[2][0])

            if func:
                func_name = func.__name__

            args = ''

        current_chain = '%s:%s(%s)' % (file_name, func_name, args)
        chain_list.append(current_chain)

    chain_list.reverse()
    return ' --> '.join(chain_list[:-2])


def get_simple_meta_data():
    frames = inspect.stack()
    try:
        obj, file_path, line_no, func_name, _, _ = frames[3]
        return (func_name + ':' + str(line_no))
    except IndexError as e:
        return ''



def bracket(regular_expression):
    return ''.join([r'(', regular_expression, r')'])


def bracket_absence(regular_expression):
    return ''.join([r'(', regular_expression, r')?'])


def absence(regular_expression):
    return ''.join([regular_expression, r'?'])


def start_end(regular_expression):
    return ''.join([r'^', regular_expression, r'$'])


import os

from datasetstation.util.log import getLogger

import boto3
from yaspin import yaspin
from yaspin.spinners import Spinners


logger = getLogger()


# All Access Permission
AWS_ACCESS_KEY_ID = "AKIAVXFFSLWHAAZD3PM6"
AWS_SECRET_ACCESS_KEY = "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"


def s3_upload_files(ds, path):
    logger.info('正在上传数据集 {}'.format(path))
    session = boto3.Session(
        aws_access_key_id=AWS_ACCESS_KEY_ID,
        aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
        region_name='cn-north-1'
    )
    s3 = session.resource('s3')
    bucket = s3.Bucket('fast-datasets')

    with yaspin(Spinners.moon, text="上传中...") as spinner:
        for subdir, _, files in os.walk(path):
            for file in files:
                full_path = os.path.join(subdir, file)
                spinner.text = full_path
                with open(full_path, 'rb') as data:
                    bucket.put_object(Key='{}/{}'.format(ds, full_path[len(path) + 1:]), Body=data)

    spinner.text = '上传完成'
    spinner.ok("✅✅✅✅✅✅✅✅✅✅")
    logger.info('上传数据集完成 {}'.format(path))




# coding:utf8

"""python util"""

import six
import json
import hashlib
from importlib import import_module
from thrift.TTornado import TTransportException

import fastweb
from fastweb.accesspoint import coroutine, Return


def head(o, h):
    return '{head}{ori}'.format(head=h, ori=o)


def write_file(filepath, content, pattern='a+'):
    with open(filepath, pattern) as f:
        f.write(content)


def filepath2pythonpath(filepath):
    if filepath.startswith('./'):
        filepath = filepath.lstrip('./')

    if filepath.endswith('/'):
        filepath = filepath.rstrip('/')

    pythonpath = filepath.replace('/', '.')
    return pythonpath


def format(st, whole=0):
    if whole:
        return '\n'.join([head(e, whole * ' ') for e in st.split('\n')])
    else:
        return st


def dumps(obj, indent, whole=0):
    if whole:
        return '\n'.join([head(e, whole * ' ') for e in json.dumps(obj, indent=indent).split('\n')])
    else:
        return json.dumps(obj, indent=indent)


def to_iter(e):
    """转换可迭代形式"""

    if isinstance(e, (six.string_types, six.string_types, six.class_types, six.text_type,
                      six.binary_type, six.class_types, six.integer_types, float)):
        return e,
    elif isinstance(e, list):
        return e
    else:
        return e


def to_plain(i):
    """转换不可迭代形式"""

    if isinstance(i, dict):
        plain = ''
        for key, value in i:
            plain += "{key}:{value}".format(key=key, value=value)
        return plain
    elif isinstance(i, (list, set)):
        return ','.join([utf8(a) for a in i])
    else:
        return i


def mixin(cls, mixcls, resume=False):
    """动态继承"""

    mixcls = to_iter(mixcls)

    if resume:
        cls.__bases__ = mixcls
    else:
        for mcls in mixcls:
            cls.__bases__ += (mcls,)


class ExceptionProcessor(object):
    """异常处理器"""

    def __init__(self, exception, processor):
        self.exception = exception
        self.processor = processor


class AsynProxyCall(object):
    """异步调用代理,用来解决__getattr__无法传递多个参数的问题"""

    def __init__(self, proxy, method, throw_exception=None, exception_processor=None):
        self.proxy = proxy
        self._method = method
        self._throw_exception = throw_exception
        self._exception_processor = exception_processor
        self._arg = None
        self._kwargs = None

    @coroutine
    def __call__(self, *arg, **kwargs):
        self._arg = arg
        self._kwargs = kwargs
        self.proxy.recorder('INFO', 'call {proxy} <{method}> start'.format(proxy=self.proxy, method=self._method))
        try:
            with fastweb.util.tool.timing('ms', 8) as t:
                ret = yield getattr(self.proxy.other, self._method)(*arg, **kwargs)
            self.proxy.recorder('INFO', 'call {proxy} <{method}> successful\n{ret} <{time}>'.format(proxy=self.proxy,
                                                                                                    method=self._method,
                                                                                                    ret=ret,
                                                                                                    time=t))
            raise Return(ret)
        except TTransportException as e:
            self.proxy.recorder('ERROR',
                                'call {proxy} <{method}> error {e} ({msg})\nreconnect'.format(proxy=self.proxy,
                                                                                              method=self._method,
                                                                                              e=type(e), msg=e))
            yield self._exception_processor.processor()
            self(*self._arg, **self._kwargs)
        else:
            raise self._throw_exception


def load_module(path):
    return import_module(path)


def load_object(path):
    """Load an object given its absolute object path, and return it.

    object can be a class, function, variable or an instance.
    path ie: 'scrapy.downloadermiddlewares.redirect.RedirectMiddleware'
    """
    try:
        dot = path.rindex('.')
    except ValueError:
        raise ValueError("Error loading object '%s': not a full path" % path)

    module, name = path[:dot], path[dot + 1:]
    mod = import_module(module)

    try:
        obj = getattr(mod, name)
    except AttributeError:
        raise NameError("Module '%s' doesn't define any object named '%s'" % (module, name))

    return obj


def merge():
    """类合并"""
    pass


def enum(**enums):
    return type('Enum', (), enums)


def isset(v):
    try:
        type(eval(v))
    except:
        return False
    else:
        return True


def md5(s):
    md = hashlib.md5()
    md.update(s)
    return md.hexdigest()


def guess_type(v):
    if v.isdigit():
        return int(v)
    else:
        return str(v)


def utf8(d):
    if isinstance(d, bytes):
        return d.decode('utf-8')
    if isinstance(d, dict):
        return dict(map(utf8, d.items()))
    if isinstance(d, tuple):
        return map(utf8, d)
    return d


def list2dict(l):
    for idx, v in enumerate(l):
        if v.isdigit():
            l[idx] = int(v)

    return dict(zip(l[0::2], l[1::2]))


def dict2list(d):
    l = []
    for k, v in zip(d.keys(), d.values()):
        l.append(str(k))
        l.append("'" + str(v) + "'")
    return l


def list2sequence(l):
    s = ''
    for idx, i in enumerate(l):
        if isinstance(i, dict):
            i = '"' + dict2sequence(i) + '"'
        s += str(i)
        if idx != len(l) - 1:
            s += ' '
    return s


def dict2sequence(d):
    return ' '.join(dict2list(d))


def sequence2dict(s):
    sq = sequence2list(utf8(s))

    if len(sq) == 1:
        return s
    else:
        return list2dict(sq)


def sequence2list(s):
    l = []
    lv = ''
    fs = False

    for w in s:
        w = utf8(w)
        if w == "'" and not fs:
            fs = True
            continue
        elif w == "'" and fs:
            fs = False
            lv and l.append(lv)
            lv = ''
            continue
        elif w == ' ' and not fs:
            lv and l.append(lv)
            lv = ''
            continue
        lv += w
    return l


def str2everything(s):
    """convert str to the most possibility type

    support type:
        int
        float
        boolean: yes, no
        str,
        list
    """

    try:
        # int, float
        c = eval(s)
        if isinstance(c, (int, float)):
            return c
        else:
            return utf8(s)
    except NameError:
        # boolean, str, list
        if s.lower() == 'yes':
            return True
        elif s.lower() == 'no':
            return False
        elif len(s.split(',')) > 1:
            return [i for i in s.split(',') if i]
        else:
            return utf8(s)
    except (SyntaxError, TypeError):
        return utf8(s)



import argparse

from datasetstation.command.sync_datasets import sync


def main():
    parser = argparse.ArgumentParser(description='china-datasets 命令行工具')
    subparsers = parser.add_subparsers(dest="subparser_name")
    sync_parser = subparsers.add_parser('sync')
    sync_parser.add_argument('-f', type=str, help='待同步的数据集配置文件')
    sync_parser.add_argument('-d', type=list, help='待同步的数据集名称')
    args = parser.parse_args()

    if args.subparser_name == 'sync':
        sync(args)


if __name__ == '__main__':
    main()

    


from .extractor import Extractor


extractor = Extractor()

clean_text = extractor.clean_text

extract_chinese = extractor.extract_chinese
extract_email = extractor.extract_email
extract_money = extractor.extract_money
extract_url = extractor.extract_url
extract_phone_number = extractor.extract_phone_number
extract_ip_address = extractor.extract_ip_address
extract_id_card = extractor.extract_id_card
extract_qq = extractor.extract_qq
extract_wechat_id = extractor.extract_wechat_id
extract_parentheses = extractor.extract_parentheses

remove_email = extractor.remove_email
remove_url = extractor.remove_url
remove_phone_number = extractor.remove_phone_number
remove_ip_address = extractor.remove_ip_address
remove_id_card = extractor.remove_id_card
remove_qq = extractor.remove_qq
remove_parentheses = extractor.remove_parentheses
remove_html_tag = extractor.remove_html_tag
remove_exception_char = extractor.remove_exception_char
remove_redundant_char = extractor.remove_redundant_char

check_chinese_char = extractor.check_chinese_char

replace_email = extractor.replace_email
replace_url = extractor.replace_url
replace_phone_number = extractor.replace_phone_number
replace_ip_address = extractor.replace_ip_address
replace_id_card = extractor.replace_id_card
replace_qq = extractor.replace_qq

del extractor


import colorlog


handler = colorlog.StreamHandler()
handler.setFormatter(
    colorlog.ColoredFormatter(
        "%(log_color)s%(levelname)-8s%(reset)s %(blue)s%(message)s",
        datefmt=None,
        reset=True,
        log_colors={
            "DEBUG": "cyan",
            "INFO": "green",
            "WARNING": "yellow",
            "ERROR": "red",
            "CRITICAL": "red,bg_white",
        },
        secondary_log_colors={},
        style="%",
    )
)


def getLogger(name="china-datasets"):
    logger = colorlog.getLogger(name)
    logger.addHandler(handler)
    logger.setLevel('INFO')
    return logger



from typing import Optional, List, Union, Dict, Tuple, Iterator, Iterable

from datasetstation.type import FileListType
from datasetstation.error import DatasetstoreFunctionNotFoundError
from datasetstation.preprocess.text.tokenizer.pretokenizer import (
    CharacterPreTokenizer,
    JiebaPreTokenizer,
)

from tokenizers.pre_tokenizers import Whitespace, PreTokenizer
from tokenizers.models import WordLevel
from tokenizers.trainers import WordLevelTrainer
from tokenizers import Tokenizer, AddedToken, PreTokenizedString, NormalizedString


class CustomTokenizer(object):

    def __init__(
        self,
        vocab: Optional[Union[str, Dict[str, int]]] = None,
        unk_token: Union[str, AddedToken] = "[UNK]",
        pre_tokenizer = None
    ):
        self.special_tokens = []
        self.special_tokens.append(unk_token)
        if vocab is not None:
            self.tokenizer = Tokenizer(WordLevel(vocab=vocab, unk_token=str(unk_token)))
        else:
            self.tokenizer = Tokenizer(WordLevel(unk_token=str(unk_token)))  # type: ignore

        if pre_tokenizer is not None:
            self.tokenizer.pre_tokenizer = PreTokenizer.custom(pre_tokenizer)  # type: ignore
        # self.tokenizer.pre_tokenizer = Whitespace()  # type: ignore
        self.trainer = WordLevelTrainer(special_tokens=self.special_tokens)

    def train(self, texts: Optional[Union[FileListType, Iterable]]):
        """对文件、文本列表进行训练
        文本应该是正常的、连续的中文，无需做任何处理
        """
        if isinstance(texts, FileListType):
            return self.tokenizer.train(texts.files, self.trainer)
        else:
            return self.tokenizer.train_from_iterator(texts, self.trainer)

    def __call__(
        self, texts: Optional[Union[str, List]]
    ) -> Optional[Union[str, List[str]]]:
        """对单文本或文本列表进行分词处理"""
        if isinstance(texts, str):
            return self.encode(texts)
        elif isinstance(texts, list):
            return [x.ids for x in self.tokenizer.encode_batch(texts)]

    def __getattr__(self, name):
        if getattr(self.tokenizer, name):
            return getattr(self.tokenizer, name)
        else:
            raise DatasetstoreFunctionNotFoundError("当前 tokenizer 不支持此方法")


class CharacterTokenizer(CustomTokenizer):
    """中文单字分词器"""

    def __init__(self, vocab: Optional[Union[str, Dict[str, int]]] = None,
        unk_token: Union[str, AddedToken] = "[UNK]"):
        super(CharacterTokenizer, self).__init__(vocab, unk_token, CharacterPreTokenizer())


class JiebaTokenizer(CustomTokenizer):
    """jieba 中文分词器"""

    def __init__(self, vocab: Optional[Union[str, Dict[str, int]]] = None,
        unk_token: Union[str, AddedToken] = "[UNK]"):
        super(JiebaTokenizer, self).__init__(vocab, unk_token, JiebaPreTokenizer())



import json

from datasetstation.util.log import getLogger
from datasetstation.util.s3 import s3_upload_files
from datasetstation.config import AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, TMP_PATH, D_S

from datasets.load import load_dataset
from datasets.filesystems.s3filesystem import S3FileSystem


logger = getLogger()


def sync(args):
    DATASETS = {}
    if args.f:
        with open(args.f) as f:
            DATASETS = json.load(f)

    # 对比数据集
    dataset_names_downloading = list(DATASETS.keys())
    dataset_names_downloading = set(map(
        lambda x: "fast-datasets/{ds}".format(ds=x), dataset_names_downloading
    ))

    s3 = S3FileSystem(
        key=AWS_ACCESS_KEY_ID,
        secret=AWS_SECRET_ACCESS_KEY,
        client_kwargs={"region_name": "cn-north-1"},
    )

    dataset_names_already = set(s3.ls("fast-datasets"))
    dataset_names_already = dataset_names_downloading - dataset_names_already

    already_num = len(dataset_names_already)
    logger.info('待同步的数据集有 {} 个\n{}'.format(already_num, D_S))

    if already_num:
        for dataset in dataset_names_already:  
            # 下载和准备数据集
            dataset_name = dataset[dataset.rfind('/')+1:]
            hg_dataset = DATASETS[dataset_name]
            dataset_path = TMP_PATH.format(ds=dataset_name)
            print('正在下载数据集 {} 到 {}'.format(hg_dataset, dataset_path))
            dataset = load_dataset(hg_dataset)
            dataset.save_to_disk(dataset_path)
            print('下载数据集 {} 到 {} 完成'.format(hg_dataset, dataset_path))
    
            # 上传数据集
            s3_upload_files(dataset_name, dataset_path)
            print('{}'.format(D_S))




# coding:utf8


import os
import uuid
import code
import atexit
import readline
from threading import Timer

from fastweb.accesspoint import ioloop, coroutine, Return

from celery import (Celery, platforms)


def get_celery_from_object(name, obj=None):
    """获取celery对象"""

    platforms.C_FORCE_ROOT = True
    celery = Celery(name)
    obj and celery.config_from_object(obj)
    return celery


def time_delay(times, delay=10, divisor=10):
    """根据次数时间延时"""

    for i in range(times):
        yield divisor * i + delay


def uniqueid(bit=64):
    """获取64位requesid"""

    return uuid.uuid1().int >> bit


class timing(object):
    """计时器"""

    __unitfactor = {'s': 1,
                    'ms': 1000,
                    'us': 1000000}

    def __init__(self, unit='s', precision=4):
        self.start = None
        self.end = None
        self.total = 0
        self.unit = unit
        self.precision = precision

    def __enter__(self):
        if self.unit not in timing.__unitfactor:
            raise KeyError('Unsupported time unit.')
        self.start = ioloop.IOLoop.current().time()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.end = ioloop.IOLoop.current().time()
        self.total = (self.end - self.start) * timing.__unitfactor[self.unit]
        self.total = round(self.total, self.precision)
        return False

    def __str__(self):
        return '{total}{unit}'.format(total=self.total, unit=self.unit)


class RetryPolicy(Exception):
    """重试策略"""

    def __init__(self, times, error, interval=0, delay=0):
        self.times = times
        self.error = error
        self.interval = interval
        self.delay = delay
        self.retry = 0


class Retry(object):
    """重试机制"""

    def __init__(self, obj, name, func, *args, **kwargs):
        self._obj = obj
        self._name = name
        self._func = func
        self._args = args
        self._kwargs = kwargs

    def run_sync(self):
        """运行重试机制"""

        try:
            return self._func(*self._args, **self._kwargs)
        except RetryPolicy as e:
            if e.retry < e.times:
                delay = e.interval * e.retry + e.delay
                e.retry += 1
                self._obj.recorder('WARN', '{name} <{retry}> retry in {delay} second...'.format(name=self._name, retry=e.retry, delay=delay))
                if delay:
                    Timer(delay, self.run).start()
                else:
                    return self.run_sync()
            else:
                self._obj.recorder('ERROR', '{name} retry error raise {exc}'.format(name=self._name, exc=e.error))
                raise e.error

    @coroutine
    def run_asyn(self):
        try:
            ret = yield self._func(*self._args, **self._kwargs)
            raise Return(ret)
        except RetryPolicy as e:
            if e.retry < e.times:
                delay = e.interval * e.retry + e.delay
                e.retry += 1
                self._obj.recorder('WARN', '{name} <{retry}> retry in {delay} second...'.format(name=self._name, retry=e.retry, delay=delay))
                if delay:
                    Timer(delay, self.run).start()
                else:
                    ret = yield self.run_asyn()
                    raise Return(ret)
            else:
                self._obj.recorder('ERROR', '{name} retry error raise {exc}'.format(name=self._name, exc=e.error))
                raise e.error


class HistoryConsole(code.InteractiveConsole):
    def __init__(self, locals=None, filename="<console>", histfile=os.path.expanduser("~/.console-history")):
        code.InteractiveConsole.__init__(self, locals, filename)
        self.init_history(histfile)

    def init_history(self, histfile):
        readline.parse_and_bind("tab: complete")
        if hasattr(readline, "read_history_file"):
            try:
                readline.read_history_file(histfile)
            except IOError:
                pass
            atexit.register(self.save_history, histfile)

    def save_history(self, histfile):
        readline.set_history_length(1000)
        readline.write_history_file(histfile)



from typing import Optional, List, Union, Dict, Tuple, Iterator, Iterable

import jieba
from tokenizers.pre_tokenizers import Whitespace, PreTokenizer
from tokenizers import Tokenizer, AddedToken, PreTokenizedString, NormalizedString


class CharacterPreTokenizer:
    def unicode_split(
        self, i: int, normalized_string: NormalizedString
    ) -> List[NormalizedString]:
        return [normalized_string[idx : idx + 1] for idx, _ in enumerate(str(normalized_string))]  # type: ignore

    def pre_tokenize(self, pretok: PreTokenizedString):
        pretok.split(self.unicode_split)

    def decode(self, tokens):
        return "".join(tokens)


class JiebaPreTokenizer:
    def jieba_split(self, i: int, normalized_string: NormalizedString) -> List[NormalizedString]:
        splits = []
        # we need to call `str(normalized_string)` because jieba expects a str,
        # not a NormalizedString
        for token, start, stop in jieba.tokenize(str(normalized_string)):
            splits.append(normalized_string[start:stop]) # type: ignore

        return splits
        # We can also easily do it in one line:
        # return [normalized_string[w[1] : w[2]] for w in jieba.tokenize(str(normalized_string))]

    def odd_number_split(
        self, i: int, normalized_string: NormalizedString
    ) -> List[NormalizedString]:
        # Just an odd example...
        splits = []
        last = 0
        for (i, char) in enumerate(str(normalized_string)):
            if char.isnumeric() and int(char) % 2 == 1:
                splits.append(normalized_string[last:i]) # type: ignore
                last = i
        # Don't forget the last one
        splits.append(normalized_string[last:]) # type: ignore
        return splits

    def pre_tokenize(self, pretok: PreTokenizedString):
        # Let's call split on the PreTokenizedString to split using `self.jieba_split`
        pretok.split(self.jieba_split)
        # Here we can call `pretok.split` multiple times if we want to apply
        # different algorithm, but we generally just need to call it once.
        pretok.split(self.odd_number_split)



import os
import pathlib

from datasetstation.backend import S3StorageBackend
from datasetstation.config import DEFAULT_DATASET_PATH

from datasets import load_from_disk


class DatasetRepo(object):

    _backend_storage = S3StorageBackend()

    def __init__(self, backend="s3"):
        self._backend = backend

    def set_backend(self, backend="s3"):
        """设置存储后端"""

        if backend == "s3":
            self._backend = backend
            self._backend_storage = S3StorageBackend()

    def list(self):
        """列出数据集仓库中的数据集名称"""
        return list(self._backend_storage.walk("fast-datasets", maxdepth=1))[0][1]

    def get_dataset_info(self, dataset):
        """获取数据集信息"""
        pass

    def _load_dataset(self, name):
        """加载数据集

        :parameter:
        name: 数据集名称

        如果本地缓存中存在，则从缓存中加载，返回 dataset
        如果本地缓存中不存在，则从网络中加载，返回 dataset
        """
        dataset_path = DEFAULT_DATASET_PATH / name
        if dataset_path.exists():
            dataset = load_from_disk(dataset_path)
            return dataset
        else:
            dataset = load_from_disk(
                self._backend_storage.backend_uri.format(ds=name),
                fs=self._backend_storage,
            )
            dataset.save_to_disk(os.fspath(DEFAULT_DATASET_PATH / name))
            return dataset

    def get(self, dataset):
        """获取数据集对象"""
        # 查看目录下的结构
        folders = list(
            self._backend_storage.walk("fast-datasets/{}".format(dataset), maxdepth=1)
        )[0][1]

        # 多数据集合加载
        if folders and "train" not in folders:
            datasets = {}
            for folder in folders:
                dataset_path = "{}/{}".format(dataset, folder)
                datasets[folder] = self._load_dataset(dataset_path)

            return datasets
        else:
            # 数据集加载
            dataset = self._load_dataset(dataset)
            return dataset

    def is_exist(self, name):
        if name in self.list():
            return True
        else:
            return False



from pathlib import Path

from datasets.arrow_dataset import Dataset
from pandas_profiling import ProfileReport


def quick_analysis(dataset, title="数据集快速分析报告", minimal=False, explorative=True, dark_mode=True, orange_mode=False):
    # config_file=Path(__file__).parent / "profiling.yml",
    df = dataset.to_pandas()
    # issue: pandas_profiling 在处理空字符串时会报错，无法继续生成报告
    df.dropna(axis=0, how="any", inplace=True)
    profile = ProfileReport(
        df,
        title=title,
        explorative=explorative,
        dark_mode=dark_mode,
        minimal=minimal,
        orange_mode=orange_mode
    )
    profile.to_notebook_iframe()
    dataset = Dataset.from_pandas(df)
    return dataset



# coding:utf8

"""进程模块"""

from multiprocessing import Process

from fastweb.util.log import recorder


class FProcess(Process):

    _fprocesses = []

    def __init__(self, name, task, *args, **kwargs):
        super(FProcess, self).__init__(name=name)
        self._task = task
        self._fprocesses.append(self)
        self._args = args
        self._kwargs = kwargs

    def __str__(self):
        return '<FProcess|{name}|{pid}>'.format(name=self.name, pid=self.pid)

    def run(self):
        recorder('INFO', '{proc} start'.format(proc=self))
        self._task(*self._args, **self._kwargs)
        recorder('INFO', '{proc} end normally'.format(proc=self))

    def stop(self, entire=False):

        if entire:
            for process in self._fprocesses:
                process.terminate()
            recorder('INFO', 'entire processes stop')
        else:
            self.terminate()
            recorder('INFO', '{proc} stop'.format(proc=self))



from typing import Optional, List, Union, Dict, Tuple, Iterator
from enum import Enum

from pydantic import BaseModel


class ExplicitEnum(str, Enum):
    """
    Enum with more explicit error message for missing values.
    """

    @classmethod
    def _missing_(cls, value):
        raise ValueError(
            f"{value} is not a valid {cls.__name__}, please select one of {list(cls._value2member_map_.keys())}"
        )


class FileListType(BaseModel):
    """文件列表类型"""
    files: List[str]






# coding=utf-8


import sys
from setuptools import setup, find_packages


kwargs = {}
install_requires = []
version = '1.0.1.9'
if sys.version_info < (3, 0):
    with open('README.md') as f:
        kwargs['long_description'] = f.read()

    with open('requirements.txt') as f:
        for require in f:
            install_requires.append(require[:-1])
    # install_requires.append('subprocess32')
elif sys.version_info > (3, 0):
    with open('README.md', encoding='utf-8') as f:
        kwargs['long_description'] = f.read()

    with open('requirements.txt', encoding='utf-8') as f:
        for require in f:
            install_requires.append(require[:-1])

# if sys.platform.startswith('linux'):
#     install_requires.append('readline')
# elif sys.platform.startswith('win'):
#     install_requires.append('pyreadline')

kwargs['install_requires'] = install_requires

setup(
    name='datasetstation',
    version=version,
    include_package_data=True,
    packages=find_packages(),
    package_data={},
    entry_points={
        'console_scripts': [
            'datasetstation = datasetstation.command.main:main',
        ],
    },
    author='CYang',
    author_email='zhangchunyang_pri@126.com',
    description="datasetstation 快速下载中文数据集，处理数据集，数据分析、可视化分析，一站式解决数据问题",
    url="https://github.com/CYang828/datasetstation",
    long_description_content_type="text/markdown",
    **kwargs
)



# coding:utf8

from threading import Thread, Event

from fastweb.util.log import recorder


class FThread(Thread):
    """封装系统thread，方便线程停止"""

    _fthreads = []

    def __init__(self, name, task, period=0, frequency=-1):
        """初始化线程

        :parameter:
         - `name`: 线程名
         - `task`: 任务函数,线程名会作为参数传递给task
         - `period`: 执行时间间隔
         - `frequency`： 执行次数，-1为永远执行，默认为永远执行
        """

        self._event = Event()
        self._period = period
        self._task = task
        self._frequency = frequency
        self._fthreads.append(self)
        super(FThread, self).__init__(name=name)

    def __str__(self):
        return '<Fthread|{name}>'.format(name=self.getName())

    def run(self):
        """运行函数,可以通过start开始线程,该函数会被自动调用"""

        while not self._event.isSet() and self._frequency:
            self._event.wait(self._period)
            self._task(self)
            self._frequency -= 1

    def join(self, timeout=0):
        """结束当前线程"""

        self._event.set()
        Thread.join(self, timeout)

    @staticmethod
    def stop(timeout=None):
        """等待所有线程执行完毕并结束线程"""

        for thread in FThread._fthreads:
            thread.join(timeout)



from __future__ import unicode_literals
import sys

import opencc

#: Character code ranges for pertinent CJK ideograph Unicode blocks.
characters = cjk_ideographs = (
    "\u3007"  # Ideographic number zero, see issue #17
    "\u4E00-\u9FFF"  # CJK Unified Ideographs
    "\u3400-\u4DBF"  # CJK Unified Ideographs Extension A
    "\uF900-\uFAFF"  # CJK Compatibility Ideographs
)
if sys.maxunicode > 0xFFFF:
    characters += (
        "\U00020000-\U0002A6DF"  # CJK Unified Ideographs Extension B
        "\U0002A700-\U0002B73F"  # CJK Unified Ideographs Extension C
        "\U0002B740-\U0002B81F"  # CJK Unified Ideographs Extension D
        "\U0002F800-\U0002FA1F"  # CJK Compatibility Ideographs Supplement
    )

#: Character code ranges for the Kangxi radicals and CJK Radicals Supplement.
radicals = "\u2F00-\u2FD5" "\u2E80-\u2EF3"  # Kangxi Radicals  # CJK Radicals Supplement

#: A string containing Chinese punctuation marks (non-stops).
non_stops = (
    # Fullwidth ASCII variants
    "\uFF02\uFF03\uFF04\uFF05\uFF06\uFF07\uFF08\uFF09\uFF0A\uFF0B\uFF0C\uFF0D"
    "\uFF0F\uFF1A\uFF1B\uFF1C\uFF1D\uFF1E\uFF20\uFF3B\uFF3C\uFF3D\uFF3E\uFF3F"
    "\uFF40\uFF5B\uFF5C\uFF5D\uFF5E\uFF5F\uFF60"
    # Halfwidth CJK punctuation
    "\uFF62\uFF63\uFF64"
    # CJK symbols and punctuation
    "\u3000\u3001\u3003"
    # CJK angle and corner brackets
    "\u3008\u3009\u300A\u300B\u300C\u300D\u300E\u300F\u3010\u3011"
    # CJK brackets and symbols/punctuation
    "\u3014\u3015\u3016\u3017\u3018\u3019\u301A\u301B\u301C\u301D\u301E\u301F"
    # Other CJK symbols
    "\u3030"
    # Special CJK indicators
    "\u303E\u303F"
    # Dashes
    "\u2013\u2014"
    # Quotation marks and apostrophe
    "\u2018\u2019\u201B\u201C\u201D\u201E\u201F"
    # General punctuation
    "\u2026\u2027"
    # Overscores and underscores
    "\uFE4F"
    # Small form variants
    "\uFE51\uFE54"
    # Latin punctuation
    "\u00B7"
)

#: A string of Chinese stops.
stops = (
    "\uFF01"  # Fullwidth exclamation mark
    "\uFF1F"  # Fullwidth question mark
    "\uFF61"  # Halfwidth ideographic full stop
    "\u3002"  # Ideographic full stop
)

#: A string containing all Chinese punctuation.
punctuation = non_stops + stops

# A sentence end is defined by a stop followed by zero or more
# container-closing marks (e.g. quotation or brackets).
_sentence_end = "[{stops}][」﹂”』’》）］｝〕〗〙〛〉】]*".format(stops=stops)

#: A regular expression pattern for a Chinese sentence. A sentence is defined
#: as a series of characters and non-stop punctuation marks followed by a stop
#: and zero or more container-closing punctuation marks (e.g. apostrophe or
# brackets).
sent = sentence = "[{characters}{radicals}{non_stops}]*{sentence_end}".format(
    characters=characters,
    radicals=radicals,
    non_stops=non_stops,
    sentence_end=_sentence_end,
)


def convert(s, mode):
    """汉字文本转换
       https://github.com/BYVoid/OpenCC
    """
    mode_dict = {
        's2t': 's2t.json',
        't2s': 't2s.json'
    }

    converter = opencc.OpenCC(mode_dict[mode])
    return converter.convert(s) 





from tqdm import tqdm
from datasets.arrow_dataset import Dataset

from collections.abc import Iterable, Callable



def preprocessing(dataset: Dataset, func_chain: Iterable = []) -> Dataset:
    pbar = tqdm(func_chain)
    for func in pbar:
        pbar.set_description('正在执行预处理 {}'.format(func.__name__))
        dataset = dataset.map(lambda x: func(x))
    pbar.set_description('预处理完成')
    return dataset


from datasetstation.repo import DatasetRepo
from datasetstation.message import CONTACT_ADMIN
from datasetstation.config import TMP_PATH
from datasetstation.util.s3 import s3_upload_files
from datasetstation.util.log import getLogger

from datasets.arrow_dataset import Dataset
from datasets.load import  load_dataset as hg_load_datasets
from yaspin import yaspin
from yaspin.spinners import Spinners


logger = getLogger()


def load_dataset(path, backend='s3', **kwargs):
    """
    Loads the dataset.
    """

    # 读取本地文件
    if path in ('csv', 'json', 'text', 'csv', 'parquet') and kwargs.get('data_files'):
        dataset = hg_load_datasets(path, **kwargs)
        return dataset
    else:
        repo = DatasetRepo()

        if not repo.is_exist(path):
            logger.warning('当前仓库不存在此数据集，{}'.format(CONTACT_ADMIN))
            return None

        with yaspin(Spinners.moon, text="下载中...") as spinner:
            try:
                dataset = repo.get(path)
                spinner.text = "下载完成"
                spinner.ok("✅✅✅✅✅✅✅✅✅✅")
                return dataset
            except BaseException as e:
                logger.error(e)
                spinner.text = "下载失败，{}".format(CONTACT_ADMIN)
                spinner.fail("🙀🙀🙀🙀🙀🙀🙀🙀🙀🙀")
                return None 


def upload_dataset(dataset, name, **dataset_info):
    # 验证数据集是否合法
    if not isinstance(dataset, Dataset):
        logger.error('当前 dataset 对象不合法，目前支持 Huggingface Dataset 对象')
        return None

    repo = DatasetRepo()
    if repo.is_exist(name):
        logger.warning('当前 dataset 名字已在仓库中存在，无需重复上传')
        return None

    # 将数据集保存到本地
    dataset_path = TMP_PATH.format(ds=name)
    dataset.save_to_disk(dataset_path)

    # 上传数据到远端
    s3_upload_files(name, dataset_path)
    return dataset


def list_datasets():
    repo = DatasetRepo()
    return repo.list()

    
